@@ -30,8 +30,8 @@ class CheckABICompliance(Function):
             aws_secret='${aws_secret}',
             bucket='mciuploads',
             content_type='text/html',
-            display_name='ABI Report:',
-            local_files_include_filter='mongoc/abi-compliance/compat_reports/**/*.html',
+            display_name='ABI Compliance Check: ',
+            local_files_include_filter='abi-compliance/compat_reports/**/compat_report.html',
             permissions='public-read',
             remote_file='mongo-c-driver/${branch_name}/${revision}/${version_id}/${build_id}/${task_id}/${execution}/abi-compliance-check/',
         ),
