@@ -7,7 +7,7 @@ from config_generator.components.funcs.find_cmake_latest import FindCMakeLatest
 
 from config_generator.etc.distros import find_large_distro
 from config_generator.etc.distros import make_distro_str
-from config_generator.etc.distros import to_cc
+from config_generator.etc.distros import compiler_to_vars
 from config_generator.etc.function import Function
 from config_generator.etc.utils import bash_exec
 
@@ -56,7 +56,7 @@ def tasks():
         distro = find_large_distro(distro_name)
 
         compile_vars = None
-        compile_vars = {'CC': to_cc(compiler)}
+        compile_vars = compiler_to_vars(compiler)
 
         if arch:
             tags.append(arch)
@@ -73,7 +73,7 @@ def tasks():
                 tags=tags,
                 commands=[
                     FindCMakeLatest.call(),
-                    ScanBuild.call(vars=compile_vars),
+                    ScanBuild.call(vars=compile_vars if compile_vars else None),
                     FunctionCall(func='upload scan artifacts'),
                 ],
             )
