@@ -21,7 +21,7 @@
 
 #include <stdio.h>
 
-#include "bcon.h"
+#include <bson/bcon.h>
 #include <bson/bson-config.h>
 
 /* These stack manipulation macros are used to manage append recursion in
@@ -38,7 +38,6 @@
 #define STACK_BSON_PARENT STACK_BSON (-1)
 #define STACK_BSON_CHILD STACK_BSON (0)
 
-#define STACK_ITER_PARENT STACK_ITER (-1)
 #define STACK_ITER_CHILD STACK_ITER (0)
 
 #define STACK_I STACK_ELE (0, i)
