@@ -24,10 +24,11 @@ test_bson_error_basic (void)
 {
    bson_error_t error;
 
-   bson_set_error (&error, 123, 456, "%s %d", "localhost", 27017);
-   BSON_ASSERT (!strcmp (error.message, "localhost 27017"));
+   bson_set_error (&error, 123, 456, "%s:%d", "localhost", 27017);
+   ASSERT_CMPSTR (error.message, "localhost:27017");
    ASSERT_CMPUINT32 (error.domain, ==, 123u);
    ASSERT_CMPUINT32 (error.code, ==, 456u);
+   ASSERT_CMPUINT (error.reserved, ==, 1u); // BSON_ERROR_CATEGORY
 }
 
 static void
@@ -39,6 +40,10 @@ test_bson_strerror_r (void)
    char *errmsg = bson_strerror_r (errno, errmsg_buf, sizeof errmsg_buf);
    // Check a message is returned. Do not check platform-dependent contents:
    ASSERT (errmsg);
+   const char *unknown_msg = "Unknown error";
+   if (strstr (errmsg, unknown_msg)) {
+      test_error ("Expected error message to contain platform-dependent content, not: '%s'", errmsg);
+   }
 }
 
 void
