@@ -1,11 +1,13 @@
-#include "mcd-rpc.h"
+#include <mlib/intencode.h>
+#include <mongoc/mcd-rpc.h>
 
 // Header-only dependency. Does NOT require linking with libmongoc.
 #define MONGOC_INSIDE
-#include "mongoc-iovec.h"
+#include <mongoc/mongoc-iovec.h>
 #undef MONGOC_INSIDE
 
 #include <bson/bson.h>
+#include <mlib/cmp.h>
 
 
 typedef struct _mcd_rpc_message_header mcd_rpc_message_header;
@@ -174,15 +176,6 @@ union _mcd_rpc_message {
    } else                                                \
       (void) 0
 
-
-static int32_t
-_int32_from_le (const void *data)
-{
-   BSON_ASSERT_PARAM (data);
-   return bson_iter_int32_unsafe (&(bson_iter_t){.raw = data});
-}
-
-
 // In addition to validating expected size against remaining bytes, ensure
 // proper conversion from little endian format.
 #define MONGOC_RPC_CONSUME(type, raw_type, from_le)                                         \
@@ -270,7 +263,7 @@ _consume_bson_objects (const uint8_t **ptr, size_t *remaining_bytes, int32_t *nu
 {
    BSON_ASSERT_PARAM (ptr);
    BSON_ASSERT_PARAM (remaining_bytes);
-   BSON_ASSERT (num_parsed || true);
+   BSON_OPTIONAL_PARAM (num_parsed);
 
    int32_t count = 0;
 
@@ -281,8 +274,7 @@ _consume_bson_objects (const uint8_t **ptr, size_t *remaining_bytes, int32_t *nu
          return false;
       }
 
-      if (doc_len < MONGOC_RPC_MINIMUM_BSON_LENGTH ||
-          bson_cmp_greater_su (doc_len, *remaining_bytes + sizeof (int32_t))) {
+      if (doc_len < MONGOC_RPC_MINIMUM_BSON_LENGTH || mlib_cmp (doc_len, >, *remaining_bytes + sizeof (int32_t))) {
          *ptr -= sizeof (int32_t); // Revert so *data_end points to start of
                                    // document as invalid input.
          return false;
@@ -364,7 +356,7 @@ _consume_op_msg_section (
 
    switch (section.kind) {
    case 0: { // Body
-      section.payload.body.section_len = _int32_from_le (*ptr);
+      section.payload.body.section_len = mlib_read_i32le (*ptr);
       section.payload.body.bson = *ptr;
 
       int32_t num_parsed = 0;
@@ -385,7 +377,7 @@ _consume_op_msg_section (
       // identifier field, but 4 bytes is sufficient to avoid unsigned integer
       // overflow when computing `remaining_section_bytes` and to encourage as
       // much progress is made parsing input data as able.
-      if (bson_cmp_less_su (section.payload.document_sequence.section_len, sizeof (int32_t))) {
+      if (mlib_cmp (section.payload.document_sequence.section_len, <, sizeof (int32_t))) {
          *ptr -= sizeof (int32_t); // Revert so *data_end points to start of
                                    // document sequence as invalid input.
          return false;
@@ -785,7 +777,7 @@ _consume_op_kill_cursors (mcd_rpc_message *rpc, const uint8_t **ptr, size_t *rem
    if (op_kill_cursors->number_of_cursor_ids < 0 ||
        // Truncation may (deliberately) leave unparsed bytes that will later
        // trigger validation failure due to unexpected remaining bytes.
-       bson_cmp_greater_su (op_kill_cursors->number_of_cursor_ids, *remaining_bytes / sizeof (int64_t))) {
+       mlib_cmp (op_kill_cursors->number_of_cursor_ids, >, *remaining_bytes / sizeof (int64_t))) {
       *ptr -= sizeof (int32_t); // Revert so *data_len points to start of
                                 // numberOfCursorIds as invalid input.
       return false;
@@ -814,7 +806,7 @@ mcd_rpc_message *
 mcd_rpc_message_from_data (const void *data, size_t length, const void **data_end)
 {
    BSON_ASSERT_PARAM (data);
-   BSON_ASSERT (data_end || true);
+   BSON_OPTIONAL_PARAM (data_end);
 
    mcd_rpc_message *rpc = bson_malloc (sizeof (mcd_rpc_message));
    mcd_rpc_message *ret = NULL;
@@ -838,7 +830,7 @@ mcd_rpc_message_from_data_in_place (mcd_rpc_message *rpc, const void *data, size
 {
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT_PARAM (data);
-   BSON_ASSERT (data_end || true);
+   BSON_OPTIONAL_PARAM (data_end);
 
    bool ret = false;
 
@@ -851,7 +843,7 @@ mcd_rpc_message_from_data_in_place (mcd_rpc_message *rpc, const void *data, size
    }
 
    if (rpc->msg_header.message_length < MONGOC_RPC_MINIMUM_MESSAGE_LENGTH ||
-       bson_cmp_greater_su (rpc->msg_header.message_length, remaining_bytes + sizeof (int32_t))) {
+       mlib_cmp (rpc->msg_header.message_length, >, remaining_bytes + sizeof (int32_t))) {
       ptr -= sizeof (int32_t); // Revert so *data_end points to start of
                                // messageLength as invalid input.
       goto fail;
@@ -1244,12 +1236,11 @@ _append_iovec_op_update (mongoc_iovec_t **iovecs,
       return false;
    }
 
-   if (!_append_iovec_data (
-          *iovecs, capacity, count, op_update->selector, (size_t) _int32_from_le (op_update->selector))) {
+   if (!_append_iovec_data (*iovecs, capacity, count, op_update->selector, mlib_read_u32le (op_update->selector))) {
       return false;
    }
 
-   if (!_append_iovec_data (*iovecs, capacity, count, op_update->update, (size_t) _int32_from_le (op_update->update))) {
+   if (!_append_iovec_data (*iovecs, capacity, count, op_update->update, mlib_read_u32le (op_update->update))) {
       return false;
    }
 
@@ -1321,7 +1312,7 @@ _append_iovec_op_query (mongoc_iovec_t **iovecs,
       return false;
    }
 
-   if (!_append_iovec_data (*iovecs, capacity, count, op_query->query, (size_t) _int32_from_le (op_query->query))) {
+   if (!_append_iovec_data (*iovecs, capacity, count, op_query->query, mlib_read_u32le (op_query->query))) {
       return false;
    }
 
@@ -1330,7 +1321,7 @@ _append_iovec_op_query (mongoc_iovec_t **iovecs,
                                capacity,
                                count,
                                op_query->return_fields_selector,
-                               (size_t) _int32_from_le (op_query->return_fields_selector))) {
+                               mlib_read_u32le (op_query->return_fields_selector))) {
          return false;
       }
    }
@@ -1401,8 +1392,7 @@ _append_iovec_op_delete (mongoc_iovec_t **iovecs,
       return false;
    }
 
-   if (!_append_iovec_data (
-          *iovecs, capacity, count, op_delete->selector, (size_t) _int32_from_le (op_delete->selector))) {
+   if (!_append_iovec_data (*iovecs, capacity, count, op_delete->selector, mlib_read_u32le (op_delete->selector))) {
       return false;
    }
 
@@ -1579,7 +1569,7 @@ _mcd_rpc_header_get_op_code_maybe_le (const mcd_rpc_message *rpc)
 
    default:
       // May be in little endian.
-      op_code = _int32_from_le (&op_code);
+      op_code = mlib_read_i32le (&op_code);
 
       switch (op_code) {
       case MONGOC_OP_CODE_COMPRESSED:
@@ -1799,7 +1789,7 @@ mcd_rpc_op_compressed_set_compressed_message (mcd_rpc_message *rpc,
 {
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT (rpc->msg_header.op_code == MONGOC_OP_CODE_COMPRESSED);
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, compressed_message_length));
+   BSON_ASSERT (mlib_in_range (int32_t, compressed_message_length));
    rpc->op_compressed.compressed_message = compressed_message;
    rpc->op_compressed.compressed_message_len = compressed_message_length;
    return (int32_t) compressed_message_length;
@@ -1826,7 +1816,7 @@ mcd_rpc_op_msg_section_get_length (const mcd_rpc_message *rpc, size_t index)
 
    switch (section->kind) {
    case 0: { // Body
-      return _int32_from_le (section->payload.body.bson);
+      return mlib_read_i32le (section->payload.body.bson);
    }
 
    case 1: { // Document Sequence
@@ -1920,7 +1910,7 @@ mcd_rpc_op_msg_section_set_identifier (mcd_rpc_message *rpc, size_t index, const
    rpc->op_msg.sections[index].payload.document_sequence.identifier = identifier;
    rpc->op_msg.sections[index].payload.document_sequence.identifier_len = identifier_len;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, identifier_len));
+   BSON_ASSERT (mlib_in_range (int32_t, identifier_len));
    return (int32_t) identifier_len;
 }
 
@@ -1932,7 +1922,7 @@ mcd_rpc_op_msg_section_set_body (mcd_rpc_message *rpc, size_t index, const void
    BSON_ASSERT (index < rpc->op_msg.sections_count);
    BSON_ASSERT (rpc->op_msg.sections[index].kind == 0);
 
-   const int32_t section_len = body ? _int32_from_le (body) : 0;
+   const int32_t section_len = body ? mlib_read_i32le (body) : 0;
 
    rpc->op_msg.sections[index].payload.body.bson = body;
    rpc->op_msg.sections[index].payload.body.section_len = section_len;
@@ -1956,7 +1946,7 @@ mcd_rpc_op_msg_section_set_document_sequence (mcd_rpc_message *rpc,
    rpc->op_msg.sections[index].payload.document_sequence.bson_objects = document_sequence;
    rpc->op_msg.sections[index].payload.document_sequence.bson_objects_len = bson_objects_len;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, document_sequence_length));
+   BSON_ASSERT (mlib_in_range (int32_t, document_sequence_length));
    return (int32_t) bson_objects_len;
 }
 
@@ -2103,7 +2093,7 @@ mcd_rpc_op_reply_set_documents (mcd_rpc_message *rpc, const void *documents, siz
    rpc->op_reply.documents = documents;
    rpc->op_reply.documents_len = documents_len;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, documents_len));
+   BSON_ASSERT (mlib_in_range (int32_t, documents_len));
    return (int32_t) documents_len;
 }
 
@@ -2151,7 +2141,7 @@ mcd_rpc_op_update_set_full_collection_name (mcd_rpc_message *rpc, const char *fu
    rpc->op_update.full_collection_name = full_collection_name;
    rpc->op_update.full_collection_name_len = length;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, length));
+   BSON_ASSERT (mlib_in_range (int32_t, length));
    return (int32_t) length;
 }
 
@@ -2168,7 +2158,7 @@ mcd_rpc_op_update_set_selector (mcd_rpc_message *rpc, const void *selector)
 {
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    rpc->op_update.selector = selector;
-   return selector ? _int32_from_le (selector) : 0;
+   return selector ? mlib_read_i32le (selector) : 0;
 }
 
 int32_t
@@ -2176,7 +2166,7 @@ mcd_rpc_op_update_set_update (mcd_rpc_message *rpc, const void *update)
 {
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    rpc->op_update.update = update;
-   return update ? _int32_from_le (update) : 0;
+   return update ? mlib_read_i32le (update) : 0;
 }
 
 
@@ -2232,7 +2222,7 @@ mcd_rpc_op_insert_set_full_collection_name (mcd_rpc_message *rpc, const char *fu
    rpc->op_insert.full_collection_name = full_collection_name;
    rpc->op_insert.full_collection_name_len = length;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, length));
+   BSON_ASSERT (mlib_in_range (int32_t, length));
    return (int32_t) length;
 }
 
@@ -2245,7 +2235,7 @@ mcd_rpc_op_insert_set_documents (mcd_rpc_message *rpc, const void *documents, si
    rpc->op_insert.documents = documents;
    rpc->op_insert.documents_len = documents_len;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, documents_len));
+   BSON_ASSERT (mlib_in_range (int32_t, documents_len));
    return (int32_t) documents_len;
 }
 
@@ -2318,7 +2308,7 @@ mcd_rpc_op_query_set_full_collection_name (mcd_rpc_message *rpc, const char *ful
    rpc->op_query.full_collection_name = full_collection_name;
    rpc->op_query.full_collection_name_len = length;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, length));
+   BSON_ASSERT (mlib_in_range (int32_t, length));
    return (int32_t) length;
 }
 
@@ -2346,7 +2336,7 @@ mcd_rpc_op_query_set_query (mcd_rpc_message *rpc, const void *query)
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT (rpc->msg_header.op_code == MONGOC_OP_CODE_QUERY);
    rpc->op_query.query = query;
-   return _int32_from_le (query);
+   return mlib_read_i32le (query);
 }
 
 int32_t
@@ -2355,7 +2345,7 @@ mcd_rpc_op_query_set_return_fields_selector (mcd_rpc_message *rpc, const void *r
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT (rpc->msg_header.op_code == MONGOC_OP_CODE_QUERY);
    rpc->op_query.return_fields_selector = return_fields_selector;
-   return return_fields_selector ? _int32_from_le (return_fields_selector) : 0;
+   return return_fields_selector ? mlib_read_i32le (return_fields_selector) : 0;
 }
 
 
@@ -2394,7 +2384,7 @@ mcd_rpc_op_get_more_set_full_collection_name (mcd_rpc_message *rpc, const char *
    rpc->op_get_more.full_collection_name = full_collection_name;
    rpc->op_get_more.full_collection_name_len = length;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, length));
+   BSON_ASSERT (mlib_in_range (int32_t, length));
    return (int32_t) length;
 }
 
@@ -2452,7 +2442,7 @@ mcd_rpc_op_delete_set_full_collection_name (mcd_rpc_message *rpc, const char *fu
    rpc->op_delete.full_collection_name = full_collection_name;
    rpc->op_delete.full_collection_name_len = length;
 
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, length));
+   BSON_ASSERT (mlib_in_range (int32_t, length));
    return (int32_t) length;
 }
 
@@ -2471,7 +2461,7 @@ mcd_rpc_op_delete_set_selector (mcd_rpc_message *rpc, const void *selector)
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT (rpc->msg_header.op_code == MONGOC_OP_CODE_DELETE);
    rpc->op_delete.selector = selector;
-   return selector ? _int32_from_le (selector) : 0;
+   return selector ? mlib_read_i32le (selector) : 0;
 }
 
 
@@ -2496,7 +2486,7 @@ mcd_rpc_op_kill_cursors_set_cursor_ids (mcd_rpc_message *rpc, const int64_t *cur
 {
    ASSERT_MCD_RPC_ACCESSOR_PRECONDITIONS;
    BSON_ASSERT (rpc->msg_header.op_code == MONGOC_OP_CODE_KILL_CURSORS);
-   BSON_ASSERT (bson_cmp_less_su (number_of_cursor_ids, (size_t) INT32_MAX / sizeof (int64_t)));
+   BSON_ASSERT (mlib_cmp (number_of_cursor_ids, <, (size_t) INT32_MAX / sizeof (int64_t)));
 
    const size_t cursor_ids_length = (size_t) number_of_cursor_ids * sizeof (int64_t);
 
