@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,14 +14,14 @@
  * limitations under the License.
  */
 
-#include "mongoc-config.h"
+#include <mongoc/mongoc-config.h>
 
 #ifdef MONGOC_ENABLE_SASL_CYRUS
-#include "mongoc-client-private.h"
-#include "mongoc-cyrus-private.h"
-#include "mongoc-cluster-cyrus-private.h"
-#include "mongoc-error.h"
-#include "mongoc-trace-private.h"
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-cyrus-private.h>
+#include <mongoc/mongoc-cluster-cyrus-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-trace-private.h>
 
 bool
 _mongoc_cluster_auth_node_cyrus (mongoc_cluster_t *cluster,
@@ -105,10 +105,10 @@ _mongoc_cluster_auth_node_cyrus (mongoc_cluster_t *cluster,
       if (!bson_iter_init_find (&iter, &reply, "payload") || !BSON_ITER_HOLDS_UTF8 (&iter)) {
          MONGOC_DEBUG ("SASL: authentication failed");
          bson_destroy (&reply);
-         bson_set_error (error,
-                         MONGOC_ERROR_CLIENT,
-                         MONGOC_ERROR_CLIENT_AUTHENTICATE,
-                         "Received invalid SASL reply from MongoDB server.");
+         _mongoc_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_AUTHENTICATE,
+                            "Received invalid SASL reply from MongoDB server.");
          goto failure;
       }
 
