@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,22 +14,20 @@
  * limitations under the License.
  */
 
-#include "mongoc-config.h"
+#include <mongoc/mongoc-config.h>
 
 #ifdef MONGOC_ENABLE_SASL_SSPI
-#include "mongoc-client-private.h"
-#include "mongoc-cluster-sspi-private.h"
-#include "mongoc-cluster-sasl-private.h"
-#include "mongoc-sasl-private.h"
-#include "mongoc-sspi-private.h"
-#include "mongoc-error.h"
-#include "mongoc-util-private.h"
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-cluster-sspi-private.h>
+#include <mongoc/mongoc-cluster-sasl-private.h>
+#include <mongoc/mongoc-sasl-private.h>
+#include <mongoc/mongoc-sspi-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-util-private.h>
 
 
 static mongoc_sspi_client_state_t *
-_mongoc_cluster_sspi_new (mongoc_uri_t *uri,
-                          mongoc_stream_t *stream,
-                          const char *hostname)
+_mongoc_cluster_sspi_new (mongoc_uri_t *uri, mongoc_stream_t *stream, const char *hostname)
 {
    WCHAR *service; /* L"serviceName@hostname@REALM" */
    const char *service_name = "mongodb";
@@ -53,31 +51,24 @@ _mongoc_cluster_sspi_new (mongoc_uri_t *uri,
    _mongoc_sasl_set_properties (&state->sasl, uri);
 
    if (state->sasl.canonicalize_host_name &&
-       _mongoc_sasl_get_canonicalized_name (
-          stream, real_name, sizeof real_name)) {
+       _mongoc_sasl_get_canonicalized_name (stream, real_name, sizeof real_name)) {
       hostname = real_name;
    }
 
    /* service realm is an SSPI-specific feature */
    if (mongoc_uri_get_mechanism_properties (uri, &properties) &&
-       bson_iter_init_find_case (&iter, &properties, "SERVICE_REALM") &&
-       BSON_ITER_HOLDS_UTF8 (&iter)) {
+       bson_iter_init_find_case (&iter, &properties, "SERVICE_REALM") && BSON_ITER_HOLDS_UTF8 (&iter)) {
       service_realm = bson_iter_utf8 (&iter, NULL);
-      service_ascii =
-         bson_strdup_printf ("%s@%s@%s", service_name, hostname, service_realm);
+      service_ascii = bson_strdup_printf ("%s@%s@%s", service_name, hostname, service_realm);
    } else {
       service_ascii = bson_strdup_printf ("%s@%s", service_name, hostname);
    }
    service_ascii_len = strlen (service_ascii);
 
    /* this is donated to the sspi */
-   service = calloc (service_ascii_len + 1, sizeof (WCHAR));
-   service_len = MultiByteToWideChar (CP_UTF8,
-                                      0,
-                                      service_ascii,
-                                      (int) service_ascii_len,
-                                      service,
-                                      (int) service_ascii_len);
+   service = bson_malloc0 ((service_ascii_len + 1) * sizeof (WCHAR));
+   service_len =
+      MultiByteToWideChar (CP_UTF8, 0, service_ascii, (int) service_ascii_len, service, (int) service_ascii_len);
    service[service_len] = L'\0';
    bson_free (service_ascii);
 
@@ -85,13 +76,8 @@ _mongoc_cluster_sspi_new (mongoc_uri_t *uri,
       tmp_creds_len = strlen (state->sasl.pass);
 
       /* this is donated to the sspi */
-      pass = calloc (tmp_creds_len + 1, sizeof (WCHAR));
-      pass_len = MultiByteToWideChar (CP_UTF8,
-                                      0,
-                                      state->sasl.pass,
-                                      (int) tmp_creds_len,
-                                      pass,
-                                      (int) tmp_creds_len);
+      pass = bson_malloc0 ((tmp_creds_len + 1) * sizeof (WCHAR));
+      pass_len = MultiByteToWideChar (CP_UTF8, 0, state->sasl.pass, (int) tmp_creds_len, pass, (int) tmp_creds_len);
       pass[pass_len] = L'\0';
    }
 
@@ -99,25 +85,13 @@ _mongoc_cluster_sspi_new (mongoc_uri_t *uri,
       tmp_creds_len = strlen (state->sasl.user);
 
       /* this is donated to the sspi */
-      user = calloc (tmp_creds_len + 1, sizeof (WCHAR));
-      user_len = MultiByteToWideChar (CP_UTF8,
-                                      0,
-                                      state->sasl.user,
-                                      (int) tmp_creds_len,
-                                      user,
-                                      (int) tmp_creds_len);
+      user = bson_malloc0 ((tmp_creds_len + 1) * sizeof (WCHAR));
+      user_len = MultiByteToWideChar (CP_UTF8, 0, state->sasl.user, (int) tmp_creds_len, user, (int) tmp_creds_len);
       user[user_len] = L'\0';
    }
 
-   res = _mongoc_sspi_auth_sspi_client_init (service,
-                                             flags,
-                                             user,
-                                             (ULONG) user_len,
-                                             NULL,
-                                             0,
-                                             pass,
-                                             (ULONG) pass_len,
-                                             state);
+   res = _mongoc_sspi_auth_sspi_client_init (
+      service, flags, user, (ULONG) user_len, NULL, 0, pass, (ULONG) pass_len, state);
 
    if (res != MONGOC_SSPI_AUTH_GSS_ERROR) {
       return state;
@@ -150,7 +124,7 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
 {
    mongoc_cmd_parts_t parts;
    mongoc_sspi_client_state_t *state;
-   SEC_CHAR* buf = NULL;
+   SEC_CHAR *buf = NULL;
    bson_iter_t iter;
    uint32_t buflen;
    bson_t reply;
@@ -161,20 +135,18 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
    int step;
    mongoc_server_stream_t *server_stream;
    bool ret = false;
+   mc_shared_tpld td = MC_SHARED_TPLD_NULL;
 
    state = _mongoc_cluster_sspi_new (cluster->uri, stream, sd->host.host);
 
    if (!state) {
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT,
-                      MONGOC_ERROR_CLIENT_AUTHENTICATE,
-                      "Couldn't initialize SSPI service.");
+      _mongoc_set_error (
+         error, MONGOC_ERROR_CLIENT, MONGOC_ERROR_CLIENT_AUTHENTICATE, "Couldn't initialize SSPI service.");
       goto failure;
    }
 
    for (step = 0;; step++) {
-      mongoc_cmd_parts_init (
-         &parts, cluster->client, "$external", MONGOC_QUERY_SLAVE_OK, &cmd);
+      mongoc_cmd_parts_init (&parts, cluster->client, "$external", MONGOC_QUERY_SECONDARY_OK, &cmd);
       parts.prohibit_lsid = true;
       bson_init (&cmd);
 
@@ -186,19 +158,13 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
 
          res = _mongoc_sspi_auth_sspi_client_unwrap (state, buf);
          response = bson_strdup (state->response);
-         _mongoc_sspi_auth_sspi_client_wrap (state,
-                                             response,
-                                             (SEC_CHAR *) state->sasl.user,
-                                             (ULONG) tmp_creds_len,
-                                             0);
+         _mongoc_sspi_auth_sspi_client_wrap (state, response, (SEC_CHAR *) state->sasl.user, (ULONG) tmp_creds_len, 0);
          bson_free (response);
       }
 
       if (res == MONGOC_SSPI_AUTH_GSS_ERROR) {
-         bson_set_error (error,
-                         MONGOC_ERROR_CLIENT,
-                         MONGOC_ERROR_CLIENT_AUTHENTICATE,
-                         "Received invalid SSPI data.");
+         _mongoc_set_error (
+            error, MONGOC_ERROR_CLIENT, MONGOC_ERROR_CLIENT_AUTHENTICATE, "Received invalid SSPI data.");
 
          mongoc_cmd_parts_cleanup (&parts);
          bson_destroy (&cmd);
@@ -206,35 +172,26 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
       }
 
       if (step == 0) {
-         _mongoc_cluster_build_sasl_start (&cmd,
-                                           "GSSAPI",
-                                           state->response,
-                                           (uint32_t) strlen (state->response));
+         _mongoc_cluster_build_sasl_start (&cmd, "GSSAPI", state->response, (uint32_t) strlen (state->response));
       } else {
          if (state->response) {
-            _mongoc_cluster_build_sasl_continue (
-               &cmd,
-               conv_id,
-               state->response,
-               (uint32_t) strlen (state->response));
+            _mongoc_cluster_build_sasl_continue (&cmd, conv_id, state->response, (uint32_t) strlen (state->response));
          } else {
             _mongoc_cluster_build_sasl_continue (&cmd, conv_id, "", 0);
          }
       }
 
-      server_stream = _mongoc_cluster_create_server_stream (
-         cluster->client->topology, sd->id, stream, error);
+      mc_tpld_renew_ref (&td, cluster->client->topology);
+      server_stream = _mongoc_cluster_create_server_stream (td.ptr, sd, stream);
 
-      if (!server_stream ||
-          !mongoc_cmd_parts_assemble (&parts, server_stream, error)) {
+      if (!mongoc_cmd_parts_assemble (&parts, server_stream, error)) {
          mongoc_server_stream_cleanup (server_stream);
          mongoc_cmd_parts_cleanup (&parts);
          bson_destroy (&cmd);
          goto failure;
       }
 
-      if (!mongoc_cluster_run_command_private (
-             cluster, &parts.assembled, &reply, error)) {
+      if (!mongoc_cluster_run_command_private (cluster, &parts.assembled, &reply, error)) {
          mongoc_server_stream_cleanup (server_stream);
          mongoc_cmd_parts_cleanup (&parts);
          bson_destroy (&cmd);
@@ -246,21 +203,19 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
       mongoc_cmd_parts_cleanup (&parts);
       bson_destroy (&cmd);
 
-      if (bson_iter_init_find (&iter, &reply, "done") &&
-          bson_iter_as_bool (&iter)) {
+      if (bson_iter_init_find (&iter, &reply, "done") && bson_iter_as_bool (&iter)) {
          bson_destroy (&reply);
          break;
       }
 
       conv_id = _mongoc_cluster_get_conversation_id (&reply);
 
-      if (!bson_iter_init_find (&iter, &reply, "payload") ||
-          !BSON_ITER_HOLDS_UTF8 (&iter)) {
+      if (!bson_iter_init_find (&iter, &reply, "payload") || !BSON_ITER_HOLDS_UTF8 (&iter)) {
          bson_destroy (&reply);
-         bson_set_error (error,
-                         MONGOC_ERROR_CLIENT,
-                         MONGOC_ERROR_CLIENT_AUTHENTICATE,
-                         "Received invalid SASL reply from MongoDB server.");
+         _mongoc_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_AUTHENTICATE,
+                            "Received invalid SASL reply from MongoDB server.");
          goto failure;
       }
 
@@ -276,6 +231,7 @@ _mongoc_cluster_auth_node_sspi (mongoc_cluster_t *cluster,
 
    ret = true;
 failure:
+   mc_tpld_drop_ref (&td);
    bson_free (buf);
    bson_free (state);
    return ret;
