@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,15 +15,17 @@
  */
 
 
-#include "mongoc-cmd-private.h"
-#include "mongoc-read-prefs-private.h"
-#include "mongoc-trace-private.h"
-#include "mongoc-client-private.h"
-#include "mongoc-read-concern-private.h"
-#include "mongoc-server-api-private.h"
-#include "mongoc-write-concern-private.h"
+#include <mlib/intencode.h>
+#include <mongoc/mongoc-cmd-private.h>
+#include <mongoc/mongoc-read-prefs-private.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-read-concern-private.h>
+#include <mongoc/mongoc-server-api-private.h>
+#include <mongoc/mongoc-write-concern-private.h>
 /* For strcasecmp on Windows */
-#include "mongoc-util-private.h"
+#include <mongoc/mongoc-util-private.h>
 
 
 void
@@ -138,7 +140,8 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts, bson_iter_t *iter, bson
          continue;
       } else if (BSON_ITER_IS_KEY (iter, "readConcern")) {
          if (!BSON_ITER_HOLDS_DOCUMENT (iter)) {
-            bson_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION, "Invalid readConcern");
+            _mongoc_set_error (
+               error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION, "Invalid readConcern");
             RETURN (false);
          }
 
@@ -164,11 +167,11 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts, bson_iter_t *iter, bson
 
       to_append = bson_iter_key (iter);
       if (!bson_append_iter (&parts->extra, to_append, -1, iter)) {
-         bson_set_error (error,
-                         MONGOC_ERROR_COMMAND,
-                         MONGOC_ERROR_COMMAND_INVALID_ARG,
-                         "Failed to append \"%s\" to create command.",
-                         to_append);
+         _mongoc_set_error (error,
+                            MONGOC_ERROR_COMMAND,
+                            MONGOC_ERROR_COMMAND_INVALID_ARG,
+                            "Failed to append \"%s\" to create command.",
+                            to_append);
          RETURN (false);
       }
    }
@@ -177,10 +180,10 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts, bson_iter_t *iter, bson
 }
 
 
-#define OPTS_ERR(_code, ...)                                                           \
-   do {                                                                                \
-      bson_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_##_code, __VA_ARGS__); \
-      RETURN (false);                                                                  \
+#define OPTS_ERR(_code, ...)                                                              \
+   do {                                                                                   \
+      _mongoc_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_##_code, __VA_ARGS__); \
+      RETURN (false);                                                                     \
    } while (0)
 
 
@@ -326,30 +329,13 @@ static void
 _mongoc_cmd_parts_add_read_prefs (bson_t *query, const mongoc_read_prefs_t *prefs)
 {
    bson_t child;
-   const char *mode_str;
-   const bson_t *tags;
-   int64_t stale;
-   const bson_t *hedge;
-
-   mode_str = _mongoc_read_mode_as_str (mongoc_read_prefs_get_mode (prefs));
-   tags = mongoc_read_prefs_get_tags (prefs);
-   stale = mongoc_read_prefs_get_max_staleness_seconds (prefs);
-   hedge = mongoc_read_prefs_get_hedge (prefs);
 
    bson_append_document_begin (query, "$readPreference", 15, &child);
-   bson_append_utf8 (&child, "mode", 4, mode_str, -1);
-   if (!bson_empty0 (tags)) {
-      bson_append_array (&child, "tags", 4, tags);
-   }
-
-   if (stale != MONGOC_NO_MAX_STALENESS) {
-      bson_append_int64 (&child, "maxStalenessSeconds", 19, stale);
-   }
-
-   if (!bson_empty0 (hedge)) {
-      bson_append_document (&child, "hedge", 5, hedge);
-   }
-
+   mongoc_read_prefs_append_contents_to_bson (
+      prefs,
+      &child,
+      MONGOC_READ_PREFS_CONTENT_FLAG_MODE | MONGOC_READ_PREFS_CONTENT_FLAG_TAGS |
+         MONGOC_READ_PREFS_CONTENT_FLAG_MAX_STALENESS_SECONDS | MONGOC_READ_PREFS_CONTENT_FLAG_HEDGE);
    bson_append_document_end (query, &child);
 }
 
@@ -390,7 +376,10 @@ _mongoc_cmd_parts_assemble_mongos (mongoc_cmd_parts_t *parts, const mongoc_serve
       max_staleness_seconds = mongoc_read_prefs_get_max_staleness_seconds (parts->read_prefs);
 
       tags = mongoc_read_prefs_get_tags (parts->read_prefs);
+      mlib_diagnostic_push ();
+      mlib_disable_deprecation_warnings ();
       hedge = mongoc_read_prefs_get_hedge (parts->read_prefs);
+      mlib_diagnostic_pop ();
    }
 
    if (server_stream->must_use_primary) {
@@ -734,11 +723,11 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts, mongoc_server_stream_t *se
     * been invalidated, error. */
    if (server_type == MONGOC_SERVER_UNKNOWN) {
       if (error) {
-         bson_set_error (error,
-                         MONGOC_ERROR_COMMAND,
-                         MONGOC_ERROR_COMMAND_INVALID_ARG,
-                         "Cannot assemble command for invalidated server: %s",
-                         server_stream->sd->error.message);
+         _mongoc_set_error (error,
+                            MONGOC_ERROR_COMMAND,
+                            MONGOC_ERROR_COMMAND_INVALID_ARG,
+                            "Cannot assemble command for invalidated server: %s",
+                            server_stream->sd->error.message);
       }
       RETURN (false);
    }
@@ -755,7 +744,7 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts, mongoc_server_stream_t *se
    cmd_name = parts->assembled.command_name = _mongoc_get_command_name (parts->assembled.command);
 
    if (!parts->assembled.command_name) {
-      bson_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, "Empty command document");
+      _mongoc_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, "Empty command document");
       GOTO (done);
    }
 
@@ -788,10 +777,10 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts, mongoc_server_stream_t *se
 
       if (cs && _mongoc_client_session_in_txn (cs)) {
          if (!IS_PREF_PRIMARY (cs->txn.opts.read_prefs) && !parts->is_write_command) {
-            bson_set_error (error,
-                            MONGOC_ERROR_TRANSACTION,
-                            MONGOC_ERROR_TRANSACTION_INVALID_STATE,
-                            "Read preference in a transaction must be primary");
+            _mongoc_set_error (error,
+                               MONGOC_ERROR_TRANSACTION,
+                               MONGOC_ERROR_TRANSACTION_INVALID_STATE,
+                               "Read preference in a transaction must be primary");
             GOTO (done);
          }
       } else if (mode != MONGOC_READ_PRIMARY && server_type != MONGOC_SERVER_STANDALONE) {
@@ -824,10 +813,10 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts, mongoc_server_stream_t *se
        */
       if (cs) {
          if (!parts->assembled.is_acknowledged) {
-            bson_set_error (error,
-                            MONGOC_ERROR_COMMAND,
-                            MONGOC_ERROR_COMMAND_INVALID_ARG,
-                            "Cannot use client session with unacknowledged command");
+            _mongoc_set_error (error,
+                               MONGOC_ERROR_COMMAND,
+                               MONGOC_ERROR_COMMAND_INVALID_ARG,
+                               "Cannot use client session with unacknowledged command");
             GOTO (done);
          }
 
@@ -879,10 +868,10 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts, mongoc_server_stream_t *se
              * than 13 before potentially appending "snapshot" read concern. */
             if (mongoc_session_opts_get_snapshot (&cs->opts) &&
                 server_stream->sd->max_wire_version < WIRE_VERSION_SNAPSHOT_READS) {
-               bson_set_error (error,
-                               MONGOC_ERROR_CLIENT,
-                               MONGOC_ERROR_CLIENT_SESSION_FAILURE,
-                               "Snapshot reads require MongoDB 5.0 or later");
+               _mongoc_set_error (error,
+                                  MONGOC_ERROR_CLIENT,
+                                  MONGOC_ERROR_CLIENT_SESSION_FAILURE,
+                                  "Snapshot reads require MongoDB 5.0 or later");
                GOTO (done);
             }
 
@@ -968,7 +957,6 @@ mongoc_cmd_is_compressible (const mongoc_cmd_t *cmd)
 void
 _mongoc_cmd_append_payload_as_array (const mongoc_cmd_t *cmd, bson_t *out)
 {
-   int32_t doc_len;
    bson_t doc;
    const uint8_t *pos;
    const char *field_name;
@@ -987,8 +975,7 @@ _mongoc_cmd_append_payload_as_array (const mongoc_cmd_t *cmd, bson_t *out)
 
       pos = cmd->payloads[i].documents;
       while (pos < cmd->payloads[i].documents + cmd->payloads[i].size) {
-         memcpy (&doc_len, pos, sizeof (doc_len));
-         doc_len = BSON_UINT32_FROM_LE (doc_len);
+         const int32_t doc_len = mlib_read_i32le (pos);
          BSON_ASSERT (bson_init_static (&doc, pos, (size_t) doc_len));
          bson_array_builder_append_document (bson, &doc);
 
