@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,14 +14,18 @@
  * limitations under the License.
  */
 
-#include "mongoc.h"
-#include "mongoc-gridfs-bucket-file-private.h"
-#include "mongoc-gridfs-bucket-private.h"
-#include "mongoc-trace-private.h"
-#include "mongoc-stream-gridfs-download-private.h"
-#include "mongoc-stream-gridfs-upload-private.h"
-#include "mongoc-collection-private.h"
-#include "mongoc-util-private.h"
+#include <mongoc/mongoc.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-gridfs-bucket-file-private.h>
+#include <mongoc/mongoc-gridfs-bucket-private.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-stream-gridfs-download-private.h>
+#include <mongoc/mongoc-stream-gridfs-upload-private.h>
+#include <mongoc/mongoc-collection-private.h>
+#include <mongoc/mongoc-util-private.h>
+#include <mlib/cmp.h>
+
+#include <inttypes.h>
 
 /* Returns the minimum of two numbers */
 static size_t
@@ -237,36 +241,36 @@ _mongoc_gridfs_bucket_read_chunk (mongoc_gridfs_bucket_file_t *file)
    }
 
    if (!r) {
-      bson_set_error (
+      _mongoc_set_error (
          &file->err, MONGOC_ERROR_GRIDFS, MONGOC_ERROR_GRIDFS_CHUNK_MISSING, "Missing chunk %d.", file->curr_chunk);
       return false;
    }
 
    r = bson_iter_init_find (&iter, next, "n");
    if (!r) {
-      bson_set_error (&file->err,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_CORRUPT,
-                      "Chunk %d missing a required field 'n'.",
-                      file->curr_chunk);
+      _mongoc_set_error (&file->err,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_CORRUPT,
+                         "Chunk %d missing a required field 'n'.",
+                         file->curr_chunk);
       return false;
    }
 
    n = bson_iter_int32 (&iter);
 
    if (n != file->curr_chunk) {
-      bson_set_error (
+      _mongoc_set_error (
          &file->err, MONGOC_ERROR_GRIDFS, MONGOC_ERROR_GRIDFS_CHUNK_MISSING, "Missing chunk %d.", file->curr_chunk);
       return false;
    }
 
    r = bson_iter_init_find (&iter, next, "data");
    if (!r) {
-      bson_set_error (&file->err,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_CORRUPT,
-                      "Chunk %d missing a required field 'data'.",
-                      file->curr_chunk);
+      _mongoc_set_error (&file->err,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_CORRUPT,
+                         "Chunk %d missing a required field 'data'.",
+                         file->curr_chunk);
       return false;
    }
 
@@ -280,13 +284,13 @@ _mongoc_gridfs_bucket_read_chunk (mongoc_gridfs_bucket_file_t *file)
    }
 
    if (data_len != expected_size) {
-      bson_set_error (&file->err,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_CORRUPT,
-                      "Chunk %d expected to have size %" PRId64 " but is size %d.",
-                      file->curr_chunk,
-                      expected_size,
-                      data_len);
+      _mongoc_set_error (&file->err,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_CORRUPT,
+                         "Chunk %d expected to have size %" PRId64 " but is size %" PRIu32 ".",
+                         file->curr_chunk,
+                         expected_size,
+                         data_len);
       return false;
    }
 
@@ -312,10 +316,10 @@ _mongoc_gridfs_bucket_file_writev (mongoc_gridfs_bucket_file_t *file, const mong
    }
 
    if (file->saved) {
-      bson_set_error (&file->err,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_PROTOCOL_ERROR,
-                      "Cannot write after saving/aborting on a GridFS file.");
+      _mongoc_set_error (&file->err,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_PROTOCOL_ERROR,
+                         "Cannot write after saving/aborting on a GridFS file.");
       return -1;
    }
 
@@ -328,7 +332,7 @@ _mongoc_gridfs_bucket_file_writev (mongoc_gridfs_bucket_file_t *file, const mong
       }
    }
 
-   BSON_ASSERT (bson_in_range_signed (size_t, file->chunk_size));
+   BSON_ASSERT (mlib_in_range (size_t, file->chunk_size));
    const size_t chunk_size = (size_t) file->chunk_size;
 
    for (size_t i = 0u; i < iovcnt; i++) {
@@ -352,7 +356,7 @@ _mongoc_gridfs_bucket_file_writev (mongoc_gridfs_bucket_file_t *file, const mong
       }
    }
 
-   BSON_ASSERT (bson_in_range_unsigned (ssize_t, total));
+   BSON_ASSERT (mlib_in_range (ssize_t, total));
    return (ssize_t) total;
 }
 
@@ -396,14 +400,14 @@ _mongoc_gridfs_bucket_file_readv (mongoc_gridfs_bucket_file_t *file, mongoc_iove
             }
             if (file->finished) {
                /* There's nothing left to read */
-               BSON_ASSERT (bson_in_range_unsigned (ssize_t, total));
+               BSON_ASSERT (mlib_in_range (ssize_t, total));
                RETURN ((ssize_t) total);
             }
          }
       }
    }
 
-   BSON_ASSERT (bson_in_range_unsigned (ssize_t, total));
+   BSON_ASSERT (mlib_in_range (ssize_t, total));
    RETURN ((ssize_t) total);
 }
 
