@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,19 +22,20 @@
 #include <time.h>
 #include <errno.h>
 
-#include "mongoc-cursor.h"
-#include "mongoc-cursor-private.h"
-#include "mongoc-collection.h"
-#include "mongoc-gridfs.h"
-#include "mongoc-gridfs-private.h"
-#include "mongoc-gridfs-file.h"
-#include "mongoc-gridfs-file-private.h"
-#include "mongoc-gridfs-file-page.h"
-#include "mongoc-gridfs-file-page-private.h"
-#include "mongoc-iovec.h"
-#include "mongoc-trace-private.h"
-#include "mongoc-util-private.h"
-#include "mongoc-error.h"
+#include <mongoc/mongoc-cursor.h>
+#include <mongoc/mongoc-cursor-private.h>
+#include <mongoc/mongoc-collection.h>
+#include <mongoc/mongoc-gridfs.h>
+#include <mongoc/mongoc-gridfs-private.h>
+#include <mongoc/mongoc-gridfs-file.h>
+#include <mongoc/mongoc-gridfs-file-private.h>
+#include <mongoc/mongoc-gridfs-file-page.h>
+#include <mongoc/mongoc-gridfs-file-page-private.h>
+#include <mongoc/mongoc-iovec.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-util-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mlib/cmp.h>
 
 static bool
 _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file);
@@ -54,40 +55,38 @@ _mongoc_gridfs_file_extend (mongoc_gridfs_file_t *file);
  * setters we need
  *****************************************************************/
 
-#define MONGOC_GRIDFS_FILE_STR_ACCESSOR(name)                             \
-   const char *mongoc_gridfs_file_get_##name (mongoc_gridfs_file_t *file) \
-   {                                                                      \
-      return file->name ? file->name : file->bson_##name;                 \
-   }                                                                      \
-   void mongoc_gridfs_file_set_##name (mongoc_gridfs_file_t *file,        \
-                                       const char *str)                   \
-   {                                                                      \
-      if (file->name) {                                                   \
-         bson_free (file->name);                                          \
-      }                                                                   \
-      file->name = bson_strdup (str);                                     \
-      file->is_dirty = 1;                                                 \
+#define MONGOC_GRIDFS_FILE_STR_ACCESSOR(name)                                       \
+   const char *mongoc_gridfs_file_get_##name (mongoc_gridfs_file_t *file)           \
+   {                                                                                \
+      return file->name ? file->name : file->bson_##name;                           \
+   }                                                                                \
+   void mongoc_gridfs_file_set_##name (mongoc_gridfs_file_t *file, const char *str) \
+   {                                                                                \
+      if (file->name) {                                                             \
+         bson_free (file->name);                                                    \
+      }                                                                             \
+      file->name = bson_strdup (str);                                               \
+      file->is_dirty = 1;                                                           \
    }
 
-#define MONGOC_GRIDFS_FILE_BSON_ACCESSOR(name)                              \
-   const bson_t *mongoc_gridfs_file_get_##name (mongoc_gridfs_file_t *file) \
-   {                                                                        \
-      if (file->name.len) {                                                 \
-         return &file->name;                                                \
-      } else if (file->bson_##name.len) {                                   \
-         return &file->bson_##name;                                         \
-      } else {                                                              \
-         return NULL;                                                       \
-      }                                                                     \
-   }                                                                        \
-   void mongoc_gridfs_file_set_##name (mongoc_gridfs_file_t *file,          \
-                                       const bson_t *bson)                  \
-   {                                                                        \
-      if (file->name.len) {                                                 \
-         bson_destroy (&file->name);                                        \
-      }                                                                     \
-      bson_copy_to (bson, &(file->name));                                   \
-      file->is_dirty = 1;                                                   \
+#define MONGOC_GRIDFS_FILE_BSON_ACCESSOR(name)                                         \
+   const bson_t *mongoc_gridfs_file_get_##name (mongoc_gridfs_file_t *file)            \
+   {                                                                                   \
+      if (file->name.len) {                                                            \
+         return &file->name;                                                           \
+      } else if (file->bson_##name.len) {                                              \
+         return &file->bson_##name;                                                    \
+      } else {                                                                         \
+         return NULL;                                                                  \
+      }                                                                                \
+   }                                                                                   \
+   void mongoc_gridfs_file_set_##name (mongoc_gridfs_file_t *file, const bson_t *bson) \
+   {                                                                                   \
+      if (file->name.len) {                                                            \
+         bson_destroy (&file->name);                                                   \
+      }                                                                                \
+      bson_copy_to (bson, &(file->name));                                              \
+      file->is_dirty = 1;                                                              \
    }
 
 MONGOC_GRIDFS_FILE_STR_ACCESSOR (md5)
@@ -105,15 +104,11 @@ MONGOC_GRIDFS_FILE_BSON_ACCESSOR (metadata)
  */
 
 bool
-mongoc_gridfs_file_set_id (mongoc_gridfs_file_t *file,
-                           const bson_value_t *id,
-                           bson_error_t *error)
+mongoc_gridfs_file_set_id (mongoc_gridfs_file_t *file, const bson_value_t *id, bson_error_t *error)
 {
    if (!file->is_dirty) {
-      bson_set_error (error,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_PROTOCOL_ERROR,
-                      "Cannot set file id after saving file.");
+      _mongoc_set_error (
+         error, MONGOC_ERROR_GRIDFS, MONGOC_ERROR_GRIDFS_PROTOCOL_ERROR, "Cannot set file id after saving file.");
       return false;
    }
    bson_value_copy (id, &file->files_id);
@@ -181,12 +176,7 @@ mongoc_gridfs_file_save (mongoc_gridfs_file_t *file)
 
    bson_append_document_end (update, &child);
 
-   r = mongoc_collection_update (file->gridfs->files,
-                                 MONGOC_UPDATE_UPSERT,
-                                 selector,
-                                 update,
-                                 NULL,
-                                 &file->error);
+   r = mongoc_collection_update (file->gridfs->files, MONGOC_UPDATE_UPSERT, selector, update, NULL, &file->error);
 
    bson_destroy (selector);
    bson_destroy (update);
@@ -428,11 +418,8 @@ mongoc_gridfs_file_destroy (mongoc_gridfs_file_t *file)
 /** readv against a gridfs file
  *  timeout_msec is unused */
 ssize_t
-mongoc_gridfs_file_readv (mongoc_gridfs_file_t *file,
-                          mongoc_iovec_t *iov,
-                          size_t iovcnt,
-                          size_t min_bytes,
-                          uint32_t timeout_msec)
+mongoc_gridfs_file_readv (
+   mongoc_gridfs_file_t *file, mongoc_iovec_t *iov, size_t iovcnt, size_t min_bytes, uint32_t timeout_msec)
 {
    uint32_t bytes_read = 0;
    int32_t r;
@@ -448,7 +435,7 @@ mongoc_gridfs_file_readv (mongoc_gridfs_file_t *file,
    BSON_ASSERT (iovcnt);
 
    /* Reading when positioned past the end does nothing */
-   if (bson_cmp_greater_equal_us (file->pos, file->length)) {
+   if (mlib_cmp (file->pos, >=, file->length)) {
       return 0;
    }
 
@@ -462,9 +449,7 @@ mongoc_gridfs_file_readv (mongoc_gridfs_file_t *file,
 
       for (;;) {
          r = _mongoc_gridfs_file_page_read (
-            file->page,
-            (uint8_t *) iov[i].iov_base + iov_pos,
-            (uint32_t) (iov[i].iov_len - iov_pos));
+            file->page, (uint8_t *) iov[i].iov_base + iov_pos, (uint32_t) (iov[i].iov_len - iov_pos));
          BSON_ASSERT (r >= 0);
 
          iov_pos += r;
@@ -474,7 +459,7 @@ mongoc_gridfs_file_readv (mongoc_gridfs_file_t *file,
          if (iov_pos == iov[i].iov_len) {
             /* filled a bucket, keep going */
             break;
-         } else if (file->length == file->pos) {
+         } else if (mlib_cmp (file->length, ==, file->pos)) {
             /* we're at the end of the file.  So we're done */
             RETURN (bytes_read);
          } else if (bytes_read >= min_bytes) {
@@ -493,10 +478,7 @@ mongoc_gridfs_file_readv (mongoc_gridfs_file_t *file,
 /** writev against a gridfs file
  *  timeout_msec is unused */
 ssize_t
-mongoc_gridfs_file_writev (mongoc_gridfs_file_t *file,
-                           const mongoc_iovec_t *iov,
-                           size_t iovcnt,
-                           uint32_t timeout_msec)
+mongoc_gridfs_file_writev (mongoc_gridfs_file_t *file, const mongoc_iovec_t *iov, size_t iovcnt, uint32_t timeout_msec)
 {
    uint32_t bytes_written = 0;
    int32_t r;
@@ -517,8 +499,7 @@ mongoc_gridfs_file_writev (mongoc_gridfs_file_t *file,
    }
 
    /* When writing past the end-of-file, fill the gap with zeros */
-   if (bson_cmp_greater_us (file->pos, file->length) &&
-       !_mongoc_gridfs_file_extend (file)) {
+   if (mlib_cmp (file->pos, >, file->length) && !_mongoc_gridfs_file_extend (file)) {
       return -1;
    }
 
@@ -532,9 +513,7 @@ mongoc_gridfs_file_writev (mongoc_gridfs_file_t *file,
 
          /* write bytes until an iov is exhausted or the page is full */
          r = _mongoc_gridfs_file_page_write (
-            file->page,
-            (uint8_t *) iov[i].iov_base + iov_pos,
-            (uint32_t) (iov[i].iov_len - iov_pos));
+            file->page, (uint8_t *) iov[i].iov_base + iov_pos, (uint32_t) (iov[i].iov_len - iov_pos));
          BSON_ASSERT (r >= 0);
 
          iov_pos += r;
@@ -585,13 +564,13 @@ _mongoc_gridfs_file_extend (mongoc_gridfs_file_t *file)
 
    BSON_ASSERT (file);
 
-   if (bson_cmp_greater_equal_su (file->length, file->pos)) {
+   if (mlib_cmp (file->length, >=, file->pos)) {
       RETURN (0);
    }
 
    const uint64_t target_length = file->pos;
 
-   BSON_ASSERT (bson_in_range_signed (uint64_t, file->length));
+   BSON_ASSERT (mlib_in_range (uint64_t, file->length));
    const uint64_t diff = file->pos - (uint64_t) file->length;
 
    if (-1 == mongoc_gridfs_file_seek (file, 0, SEEK_END)) {
@@ -606,9 +585,8 @@ _mongoc_gridfs_file_extend (mongoc_gridfs_file_t *file)
       /* Set bytes until we reach the limit or fill a page */
       {
          const uint64_t len = target_length - file->pos;
-         BSON_ASSERT (bson_in_range_unsigned (uint32_t, len));
-         file->pos +=
-            _mongoc_gridfs_file_page_memset0 (file->page, (uint32_t) len);
+         BSON_ASSERT (mlib_in_range (uint32_t, len));
+         file->pos += _mongoc_gridfs_file_page_memset0 (file->page, (uint32_t) len);
       }
 
       if (file->pos == target_length) {
@@ -620,11 +598,11 @@ _mongoc_gridfs_file_extend (mongoc_gridfs_file_t *file)
       }
    }
 
-   BSON_ASSERT (bson_in_range_unsigned (int64_t, target_length));
+   BSON_ASSERT (mlib_in_range (int64_t, target_length));
    file->length = (int64_t) target_length;
    file->is_dirty = true;
 
-   BSON_ASSERT (bson_in_range_unsigned (ssize_t, diff));
+   BSON_ASSERT (mlib_in_range (ssize_t, diff));
    RETURN ((ssize_t) diff);
 }
 
@@ -669,12 +647,7 @@ _mongoc_gridfs_file_flush_page (mongoc_gridfs_file_t *file)
    bson_append_int32 (update, "n", -1, file->n);
    bson_append_binary (update, "data", -1, BSON_SUBTYPE_BINARY, buf, len);
 
-   r = mongoc_collection_update (file->gridfs->chunks,
-                                 MONGOC_UPDATE_UPSERT,
-                                 selector,
-                                 update,
-                                 NULL,
-                                 &file->error);
+   r = mongoc_collection_update (file->gridfs->chunks, MONGOC_UPDATE_UPSERT, selector, update, NULL, &file->error);
 
    bson_destroy (selector);
    bson_destroy (update);
@@ -737,11 +710,8 @@ divide_round_up (int64_t num, int64_t denom)
 static void
 missing_chunk (mongoc_gridfs_file_t *file)
 {
-   bson_set_error (&file->error,
-                   MONGOC_ERROR_GRIDFS,
-                   MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
-                   "missing chunk number %" PRId32,
-                   file->n);
+   _mongoc_set_error (
+      &file->error, MONGOC_ERROR_GRIDFS, MONGOC_ERROR_GRIDFS_CHUNK_MISSING, "missing chunk number %" PRId32, file->n);
 
    if (file->cursor) {
       mongoc_cursor_destroy (file->cursor);
@@ -774,16 +744,8 @@ missing_chunk (mongoc_gridfs_file_t *file)
 static bool
 _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
 {
-   bson_t query;
-   bson_t child;
-   bson_t opts;
-   const char *key;
-   bson_iter_t iter;
-   int64_t existing_chunks;
-   int64_t required_chunks;
-
    const uint8_t *data = NULL;
-   uint32_t len;
+   uint32_t len = 0u;
 
    ENTRY;
 
@@ -798,8 +760,9 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
 
    /* if the file pointer is past the end of the current file (i.e. pointing to
     * a new chunk), we'll pass the page constructor a new empty page. */
-   existing_chunks = divide_round_up (file->length, file->chunk_size);
-   required_chunks = divide_round_up (file->pos + 1, file->chunk_size);
+   const int64_t existing_chunks = divide_round_up (file->length, file->chunk_size);
+   const int64_t required_chunks = divide_round_up (file->pos + 1, file->chunk_size);
+
    if (required_chunks > existing_chunks) {
       data = (uint8_t *) "";
       len = 0;
@@ -812,12 +775,16 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
       }
 
       if (!file->cursor) {
+         bson_t query;
          bson_init (&query);
          BSON_APPEND_VALUE (&query, "files_id", &file->files_id);
+
+         bson_t child;
          BSON_APPEND_DOCUMENT_BEGIN (&query, "n", &child);
          BSON_APPEND_INT32 (&child, "$gte", file->n);
          bson_append_document_end (&query, &child);
 
+         bson_t opts;
          bson_init (&opts);
          BSON_APPEND_DOCUMENT_BEGIN (&opts, "sort", &child);
          BSON_APPEND_INT32 (&child, "n", 1);
@@ -830,8 +797,7 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
          bson_append_document_end (&opts, &child);
 
          /* find all chunks greater than or equal to our current file pos */
-         file->cursor = mongoc_collection_find_with_opts (
-            file->gridfs->chunks, &query, &opts, NULL);
+         file->cursor = mongoc_collection_find_with_opts (file->gridfs->chunks, &query, &opts, NULL);
 
          file->cursor_range[0] = file->n;
          file->cursor_range[1] = (uint32_t) (file->length / file->chunk_size);
@@ -846,7 +812,7 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
 
       /* we might have had a cursor before, then seeked ahead past a chunk.
        * iterate until we're on the right chunk */
-      while (bson_cmp_less_equal_us (file->cursor_range[0], file->n)) {
+      while (mlib_cmp (file->cursor_range[0], <=, file->n)) {
          if (!mongoc_cursor_next (file->cursor, &chunk)) {
             /* copy cursor error; if there's none, we're missing a chunk */
             if (!mongoc_cursor_error (file->cursor, &file->error)) {
@@ -859,11 +825,12 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
          file->cursor_range[0]++;
       }
 
+      bson_iter_t iter;
       BSON_ASSERT (bson_iter_init (&iter, chunk));
 
       /* grab out what we need from the chunk */
       while (bson_iter_next (&iter)) {
-         key = bson_iter_key (&iter);
+         const char *const key = bson_iter_key (&iter);
 
          if (strcmp (key, "n") == 0) {
             if (file->n != bson_iter_int32 (&iter)) {
@@ -872,40 +839,52 @@ _mongoc_gridfs_file_refresh_page (mongoc_gridfs_file_t *file)
             }
          } else if (strcmp (key, "data") == 0) {
             bson_iter_binary (&iter, NULL, &len, &data);
+            // If this not the last chunk, ensure length is equal to chunk size.
+            bool is_last_chunk = ((file->n + 1) == existing_chunks);
+            // If this is not the last chunk, error.
+            if (!is_last_chunk && mlib_cmp (len, !=, file->chunk_size)) {
+               _mongoc_set_error (&file->error,
+                                  MONGOC_ERROR_GRIDFS,
+                                  MONGOC_ERROR_GRIDFS_CORRUPT,
+                                  "corrupt chunk number %" PRId32 ": not equal to chunk size: %" PRId32,
+                                  file->n,
+                                  file->chunk_size);
+               RETURN (0);
+            }
          } else {
             /* Unexpected key. This should never happen */
             RETURN (0);
          }
       }
 
-      if (file->n != file->pos / file->chunk_size) {
+      if (mlib_cmp (file->n, !=, file->pos / file->chunk_size)) {
          return 0;
       }
    }
 
    if (!data) {
-      bson_set_error (&file->error,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
-                      "corrupt chunk number %" PRId32,
-                      file->n);
+      _mongoc_set_error (&file->error,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
+                         "corrupt chunk number %" PRId32 ": no data",
+                         file->n);
       RETURN (0);
    }
 
-   if (bson_cmp_greater_us (len, file->chunk_size)) {
-      bson_set_error (&file->error,
-                      MONGOC_ERROR_GRIDFS,
-                      MONGOC_ERROR_GRIDFS_CORRUPT,
-                      "corrupt chunk number %" PRId32 ": bad size",
-                      file->n);
+   if (mlib_cmp (len, >, file->chunk_size)) {
+      _mongoc_set_error (&file->error,
+                         MONGOC_ERROR_GRIDFS,
+                         MONGOC_ERROR_GRIDFS_CORRUPT,
+                         "corrupt chunk number %" PRId32 ": greater than chunk size: %" PRId32,
+                         file->n,
+                         file->chunk_size);
       RETURN (0);
    }
 
    file->page = _mongoc_gridfs_file_page_new (data, len, file->chunk_size);
 
    /* seek in the page towards wherever we're supposed to be */
-   RETURN (
-      _mongoc_gridfs_file_page_seek (file->page, file->pos % file->chunk_size));
+   RETURN (_mongoc_gridfs_file_page_seek (file->page, file->pos % file->chunk_size));
 }
 
 
@@ -952,7 +931,7 @@ mongoc_gridfs_file_seek (mongoc_gridfs_file_t *file, int64_t delta, int whence)
       offset = delta;
       break;
    case SEEK_CUR:
-      BSON_ASSERT (bson_in_range_unsigned (int64_t, file->pos));
+      BSON_ASSERT (mlib_in_range (int64_t, file->pos));
       offset = (int64_t) file->pos + delta;
       break;
    case SEEK_END:
@@ -961,8 +940,6 @@ mongoc_gridfs_file_seek (mongoc_gridfs_file_t *file, int64_t delta, int whence)
    default:
       errno = EINVAL;
       return -1;
-
-      break;
    }
 
    if (offset < 0) {
@@ -988,15 +965,15 @@ mongoc_gridfs_file_seek (mongoc_gridfs_file_t *file, int64_t delta, int whence)
        * lazily load */
    } else if (file->page) {
       const int64_t n = offset % file->chunk_size;
-      BSON_ASSERT (bson_in_range_signed (uint32_t, n));
+      BSON_ASSERT (mlib_in_range (uint32_t, n));
       BSON_ASSERT (_mongoc_gridfs_file_page_seek (file->page, (uint32_t) n));
    }
 
    file->pos = (uint64_t) offset;
 
-   BSON_ASSERT (bson_in_range_signed (uint64_t, file->chunk_size));
+   BSON_ASSERT (mlib_in_range (uint64_t, file->chunk_size));
    const uint64_t n = file->pos / (uint64_t) file->chunk_size;
-   BSON_ASSERT (bson_in_range_unsigned (int32_t, n));
+   BSON_ASSERT (mlib_in_range (int32_t, n));
    file->n = (int32_t) n;
 
    return 0;
@@ -1017,11 +994,9 @@ mongoc_gridfs_file_error (mongoc_gridfs_file_t *file, bson_error_t *error)
    BSON_ASSERT (error);
 
    if (BSON_UNLIKELY (file->error.domain)) {
-      bson_set_error (error,
-                      file->error.domain,
-                      file->error.code,
-                      "%s",
-                      file->error.message);
+      if (error) {
+         *error = file->error;
+      }
       RETURN (true);
    }
 
@@ -1070,16 +1045,14 @@ mongoc_gridfs_file_remove (mongoc_gridfs_file_t *file, bson_error_t *error)
 
    BSON_APPEND_VALUE (&sel, "_id", &file->files_id);
 
-   if (!mongoc_collection_delete_one (
-          file->gridfs->files, &sel, NULL, NULL, error)) {
+   if (!mongoc_collection_delete_one (file->gridfs->files, &sel, NULL, NULL, error)) {
       goto cleanup;
    }
 
    bson_reinit (&sel);
    BSON_APPEND_VALUE (&sel, "files_id", &file->files_id);
 
-   if (!mongoc_collection_delete_many (
-          file->gridfs->chunks, &sel, NULL, NULL, error)) {
+   if (!mongoc_collection_delete_many (file->gridfs->chunks, &sel, NULL, NULL, error)) {
       goto cleanup;
    }
 
