@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,22 +18,21 @@
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "gridfs"
 
-#include "mongoc-bulk-operation.h"
-#include "mongoc-client-private.h"
-#include "mongoc-collection.h"
-#include "mongoc-collection-private.h"
-#include "mongoc-error.h"
-#include "mongoc-index.h"
-#include "mongoc-gridfs.h"
-#include "mongoc-gridfs-private.h"
-#include "mongoc-gridfs-file.h"
-#include "mongoc-gridfs-file-private.h"
-#include "mongoc-gridfs-file-list.h"
-#include "mongoc-gridfs-file-list-private.h"
-#include "mongoc-client.h"
-#include "mongoc-trace-private.h"
-#include "mongoc-cursor-private.h"
-#include "mongoc-util-private.h"
+#include <mongoc/mongoc-bulk-operation.h>
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-collection.h>
+#include <mongoc/mongoc-collection-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-gridfs.h>
+#include <mongoc/mongoc-gridfs-private.h>
+#include <mongoc/mongoc-gridfs-file.h>
+#include <mongoc/mongoc-gridfs-file-private.h>
+#include <mongoc/mongoc-gridfs-file-list.h>
+#include <mongoc/mongoc-gridfs-file-list-private.h>
+#include <mongoc/mongoc-client.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-cursor-private.h>
+#include <mongoc/mongoc-util-private.h>
 
 #define MONGOC_GRIDFS_STREAM_CHUNK 4096
 
@@ -116,10 +115,13 @@ _mongoc_gridfs_new (mongoc_client_t *client, const char *db, const char *prefix,
 
    gridfs->client = client;
 
-   bson_snprintf (buf, sizeof (buf), "%s.chunks", prefix);
+   // Expect no truncation from above, checking no error occurred.
+   int req = bson_snprintf (buf, sizeof (buf), "%s.chunks", prefix);
+   BSON_ASSERT (req > 0);
    gridfs->chunks = mongoc_client_get_collection (client, db, buf);
 
-   bson_snprintf (buf, sizeof (buf), "%s.files", prefix);
+   req = bson_snprintf (buf, sizeof (buf), "%s.files", prefix);
+   BSON_ASSERT (req > 0);
    gridfs->files = mongoc_client_get_collection (client, db, buf);
 
    r = _mongoc_gridfs_ensure_index (gridfs, error);
@@ -172,37 +174,6 @@ mongoc_gridfs_destroy (mongoc_gridfs_t *gridfs)
 }
 
 
-/** find all matching gridfs files */
-mongoc_gridfs_file_list_t *
-mongoc_gridfs_find (mongoc_gridfs_t *gridfs, const bson_t *query)
-{
-   return _mongoc_gridfs_file_list_new (gridfs, query, 0);
-}
-
-
-/** find a single gridfs file */
-mongoc_gridfs_file_t *
-mongoc_gridfs_find_one (mongoc_gridfs_t *gridfs, const bson_t *query, bson_error_t *error)
-{
-   mongoc_gridfs_file_list_t *list;
-   mongoc_gridfs_file_t *file;
-
-   ENTRY;
-
-   list = _mongoc_gridfs_file_list_new (gridfs, query, 1);
-
-   file = mongoc_gridfs_file_list_next (list);
-   if (!mongoc_gridfs_file_list_error (list, error) && error) {
-      /* no error, but an error out-pointer was provided - clear it */
-      memset (error, 0, sizeof (*error));
-   }
-
-   mongoc_gridfs_file_list_destroy (list);
-
-   RETURN (file);
-}
-
-
 /** find all matching gridfs files */
 mongoc_gridfs_file_list_t *
 mongoc_gridfs_find_with_opts (mongoc_gridfs_t *gridfs, const bson_t *filter, const bson_t *opts)
@@ -382,7 +353,7 @@ mongoc_gridfs_remove_by_filename (mongoc_gridfs_t *gridfs, const char *filename,
    BSON_ASSERT (gridfs);
 
    if (!filename) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_GRIDFS, MONGOC_ERROR_GRIDFS_INVALID_FILENAME, "A non-NULL filename must be specified.");
       return false;
    }
