@@ -61,8 +61,8 @@ typedef enum {
    MONGOC_MD_FLAG_EXPERIMENTAL_FEATURES = 10,
    MONGOC_MD_FLAG_HAVE_SASL_CLIENT_DONE = 11,
    MONGOC_MD_FLAG_HAVE_WEAK_SYMBOLS = 12,
-   MONGOC_MD_FLAG_NO_AUTOMATIC_GLOBALS = 13,
-   MONGOC_MD_FLAG_ENABLE_SSL_LIBRESSL = 14,
+   MONGOC_MD_FLAG_NO_AUTOMATIC_GLOBALS_UNUSED = 13, // Removed in CDRIVER-1330.
+   MONGOC_MD_FLAG_ENABLE_SSL_LIBRESSL_UNUSED = 14,  // Removed in CDRIVER-5693.
    MONGOC_MD_FLAG_ENABLE_SASL_CYRUS = 15,
    MONGOC_MD_FLAG_ENABLE_SASL_SSPI = 16,
    MONGOC_MD_FLAG_HAVE_SOCKLEN = 17,
