@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,14 +14,17 @@
  * limitations under the License.
  */
 
-#include "mongoc-http-private.h"
+#include <mongoc/mongoc-http-private.h>
 
-#include "mongoc-client-private.h"
-#include "mongoc-host-list-private.h"
-#include "mongoc-stream-tls.h"
-#include "mongoc-stream-private.h"
-#include "mongoc-buffer-private.h"
-#include "mcd-time.h"
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-host-list-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-stream-tls.h>
+#include <mongoc/mongoc-stream-private.h>
+#include <mongoc/mongoc-buffer-private.h>
+#include <mongoc/mcd-time.h>
+#include <common-string-private.h>
+#include <mlib/cmp.h>
 
 void
 _mongoc_http_request_init (mongoc_http_request_t *request)
@@ -45,53 +48,50 @@ _mongoc_http_response_cleanup (mongoc_http_response_t *response)
    bson_free (response->body);
 }
 
-bson_string_t *
-_mongoc_http_render_request_head (const mongoc_http_request_t *req)
+void
+_mongoc_http_render_request_head (mcommon_string_append_t *append, const mongoc_http_request_t *req)
 {
    BSON_ASSERT_PARAM (req);
-   char *path = NULL;
+
+   mcommon_string_append_printf (append, "%s ", req->method);
 
    // Default paths
    if (!req->path) {
       // Top path:
-      path = bson_strdup ("/");
+      mcommon_string_append (append, "/");
    } else if (req->path[0] != '/') {
       // Path MUST be prefixed with a separator
-      path = bson_strdup_printf ("/%s", req->path);
+      mcommon_string_append (append, "/");
+      mcommon_string_append (append, req->path);
    } else {
       // Just copy the path
-      path = bson_strdup (req->path);
+      mcommon_string_append (append, req->path);
    }
 
-   bson_string_t *const string = bson_string_new ("");
-   // Set the request line
-   bson_string_append_printf (string, "%s %s HTTP/1.0\r\n", req->method, path);
-   // (We're done with the path string:)
-   bson_free (path);
+   mcommon_string_append (append, " HTTP/1.0\r\n");
 
    /* Always add Host header. */
-   bson_string_append_printf (string, "Host: %s:%d\r\n", req->host, req->port);
+   mcommon_string_append_printf (append, "Host: %s:%d\r\n", req->host, req->port);
    /* Always add Connection: close header to ensure server closes connection. */
-   bson_string_append_printf (string, "Connection: close\r\n");
+   mcommon_string_append (append, "Connection: close\r\n");
    /* Add Content-Length if body is included. */
    if (req->body_len) {
-      bson_string_append_printf (string, "Content-Length: %d\r\n", req->body_len);
+      mcommon_string_append_printf (append, "Content-Length: %d\r\n", req->body_len);
    }
    // Add any extra headers
    if (req->extra_headers) {
-      bson_string_append (string, req->extra_headers);
+      mcommon_string_append (append, req->extra_headers);
    }
 
    // Final terminator
-   bson_string_append (string, "\r\n");
-   return string;
+   mcommon_string_append (append, "\r\n");
 }
 
 static int32_t
 _mongoc_http_msec_remaining (mcd_timer timer)
 {
    const int64_t msec = mcd_get_milliseconds (mcd_timer_remaining (timer));
-   BSON_ASSERT (bson_in_range_signed (int32_t, msec));
+   BSON_ASSERT (mlib_in_range (int32_t, msec));
    return (int32_t) msec;
 }
 
@@ -108,7 +108,6 @@ _mongoc_http_send (const mongoc_http_request_t *req,
    bool ret = false;
    mongoc_iovec_t iovec;
    char *path = NULL;
-   bson_string_t *http_request = NULL;
    mongoc_buffer_t http_response_buf;
    char *http_response_str;
    char *ptr;
@@ -116,6 +115,9 @@ _mongoc_http_send (const mongoc_http_request_t *req,
 
    const mcd_timer timer = mcd_timer_expire_after (mcd_milliseconds (timeout_ms));
 
+   mcommon_string_append_t http_request;
+   mcommon_string_new_as_append (&http_request);
+
    memset (res, 0, sizeof (*res));
    _mongoc_buffer_init (&http_response_buf, NULL, 0, NULL, NULL);
 
@@ -129,17 +131,17 @@ _mongoc_http_send (const mongoc_http_request_t *req,
       &host_list,
       error);
    if (!stream) {
-      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Failed to connect to: %s", req->host);
+      _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Failed to connect to: %s", req->host);
       goto fail;
    }
 
 #ifndef MONGOC_ENABLE_SSL
    if (use_tls) {
-      bson_set_error (error,
-                      MONGOC_ERROR_STREAM,
-                      MONGOC_ERROR_STREAM_SOCKET,
-                      "Failed to connect to %s: libmongoc not built with TLS support",
-                      req->host);
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_STREAM,
+                         MONGOC_ERROR_STREAM_SOCKET,
+                         "Failed to connect to %s: libmongoc not built with TLS support",
+                         req->host);
       goto fail;
    }
 #else
@@ -149,7 +151,7 @@ _mongoc_http_send (const mongoc_http_request_t *req,
       BSON_ASSERT (ssl_opts);
       tls_stream = mongoc_stream_tls_new_with_hostname (stream, req->host, ssl_opts, true);
       if (!tls_stream) {
-         bson_set_error (
+         _mongoc_set_error (
             error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Failed create TLS stream to: %s", req->host);
          goto fail;
       }
@@ -169,9 +171,10 @@ _mongoc_http_send (const mongoc_http_request_t *req,
       path = bson_strdup (req->path);
    }
 
-   http_request = _mongoc_http_render_request_head (req);
-   iovec.iov_base = http_request->str;
-   iovec.iov_len = http_request->len;
+   _mongoc_http_render_request_head (&http_request, req);
+
+   iovec.iov_base = mcommon_str_from_append (&http_request);
+   iovec.iov_len = mcommon_strlen_from_append (&http_request);
 
    if (!_mongoc_stream_writev_full (stream, &iovec, 1, _mongoc_http_msec_remaining (timer), error)) {
       goto fail;
@@ -196,18 +199,19 @@ _mongoc_http_send (const mongoc_http_request_t *req,
          break;
       }
       if (http_response_buf.len > 1024 * 1024 * 8) {
-         bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "HTTP response message is too large");
+         _mongoc_set_error (
+            error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "HTTP response message is too large");
          goto fail;
       }
    }
 
    if (mongoc_stream_timed_out (stream)) {
-      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Timeout reading from stream");
+      _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Timeout reading from stream");
       goto fail;
    }
 
    if (http_response_buf.len == 0) {
-      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "No response received");
+      _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "No response received");
       goto fail;
    }
 
@@ -223,12 +227,12 @@ _mongoc_http_send (const mongoc_http_request_t *req,
    }
 
    if (!ptr) {
-      bson_set_error (error,
-                      MONGOC_ERROR_STREAM,
-                      MONGOC_ERROR_STREAM_SOCKET,
-                      "No HTTP version leader in HTTP response. Expected '%s' or '%s'",
-                      proto_leader_10,
-                      proto_leader_11);
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_STREAM,
+                         MONGOC_ERROR_STREAM_SOCKET,
+                         "No HTTP version leader in HTTP response. Expected '%s' or '%s'",
+                         proto_leader_10,
+                         proto_leader_11);
       goto fail;
    }
 
@@ -236,7 +240,7 @@ _mongoc_http_send (const mongoc_http_request_t *req,
    ptr += strlen (proto_leader_10);
    ssize_t remain = resp_end_ptr - ptr;
    if (remain < 4) {
-      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Short read in HTTP response");
+      _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "Short read in HTTP response");
       goto fail;
    }
 
@@ -245,30 +249,30 @@ _mongoc_http_send (const mongoc_http_request_t *req,
    char *status_endptr;
    res->status = strtol (status_buf, &status_endptr, 10);
    if (status_endptr != status_buf + 3) {
-      bson_set_error (error,
-                      MONGOC_ERROR_STREAM,
-                      MONGOC_ERROR_STREAM_SOCKET,
-                      "Invalid HTTP response status string %*.s",
-                      4,
-                      status_buf);
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_STREAM,
+                         MONGOC_ERROR_STREAM_SOCKET,
+                         "Invalid HTTP response status string %*.s",
+                         4,
+                         status_buf);
       goto fail;
    }
 
    /* Find the end of the headers. */
    ptr = strstr (http_response_str, header_delimiter);
    if (NULL == ptr) {
-      bson_set_error (error,
-                      MONGOC_ERROR_STREAM,
-                      MONGOC_ERROR_STREAM_SOCKET,
-                      "Error occurred reading response: end of headers not found");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_STREAM,
+                         MONGOC_ERROR_STREAM_SOCKET,
+                         "Error occurred reading response: end of headers not found");
       goto fail;
    }
 
    const size_t headers_len = (size_t) (ptr - http_response_str);
-   BSON_ASSERT (bson_in_range_unsigned (int, headers_len));
+   BSON_ASSERT (mlib_in_range (int, headers_len));
 
    const size_t body_len = http_response_buf.len - headers_len - strlen (header_delimiter);
-   BSON_ASSERT (bson_in_range_unsigned (int, body_len));
+   BSON_ASSERT (mlib_in_range (int, body_len));
 
    res->headers_len = (int) headers_len;
    res->headers = bson_strndup (http_response_str, (size_t) headers_len);
@@ -280,9 +284,7 @@ _mongoc_http_send (const mongoc_http_request_t *req,
 
 fail:
    mongoc_stream_destroy (stream);
-   if (http_request) {
-      bson_string_free (http_request, true);
-   }
+   mcommon_string_from_append_destroy (&http_request);
    _mongoc_buffer_destroy (&http_response_buf);
    bson_free (path);
    return ret;
