@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,10 +15,11 @@
  */
 
 
-#include "mongoc-rpc-private.h"
+#include <mongoc/mongoc-rpc-private.h>
 
-#include "mongoc-counters-private.h"
-#include "mongoc-trace-private.h"
+#include <mongoc/mongoc-counters-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-trace-private.h>
 
 
 bool
@@ -126,6 +127,30 @@ _parse_error_reply (const bson_t *doc, bool check_wce, uint32_t *code, const cha
 }
 
 
+static void
+_mongoc_cmd_set_error (bson_error_t *error, int32_t error_api_version, uint32_t code, const char *msg)
+{
+   BSON_OPTIONAL_PARAM (error);
+   BSON_ASSERT_PARAM (msg);
+
+   uint8_t category = MONGOC_ERROR_CATEGORY_SERVER;
+
+   if (code == MONGOC_ERROR_PROTOCOL_ERROR) {
+      // Map protocolError to commandNotFound for backward compatibility (DRIVERS-192).
+      code = MONGOC_ERROR_QUERY_COMMAND_NOT_FOUND;
+   } else if (code == 0) {
+      // Reply was "not OK" but no error code was provided. Use our own error code.
+      code = MONGOC_ERROR_QUERY_FAILURE;
+      category = MONGOC_ERROR_CATEGORY;
+   }
+
+   const mongoc_error_domain_t domain =
+      error_api_version >= MONGOC_ERROR_API_VERSION_2 ? MONGOC_ERROR_SERVER : MONGOC_ERROR_QUERY;
+
+   _mongoc_set_error_with_category (error, category, domain, code, "%s", msg);
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -147,32 +172,27 @@ _parse_error_reply (const bson_t *doc, bool check_wce, uint32_t *code, const cha
 bool
 _mongoc_cmd_check_ok (const bson_t *doc, int32_t error_api_version, bson_error_t *error)
 {
-   mongoc_error_domain_t domain =
-      error_api_version >= MONGOC_ERROR_API_VERSION_2 ? MONGOC_ERROR_SERVER : MONGOC_ERROR_QUERY;
-   uint32_t code;
-   bson_iter_t iter;
-   const char *msg = "Unknown command error";
-
    ENTRY;
 
    BSON_ASSERT (doc);
 
-   if (bson_iter_init_find (&iter, doc, "ok") && bson_iter_as_bool (&iter)) {
-      /* no error */
-      RETURN (true);
+   {
+      bson_iter_t iter;
+
+      if (bson_iter_init_find (&iter, doc, "ok") && bson_iter_as_bool (&iter)) {
+         /* no error */
+         RETURN (true);
+      }
    }
 
+   uint32_t code;
+   const char *msg = "Unknown command error";
+
    if (!_parse_error_reply (doc, false /* check_wce */, &code, &msg)) {
       RETURN (true);
    }
 
-   if (code == MONGOC_ERROR_PROTOCOL_ERROR || code == 13390) {
-      code = MONGOC_ERROR_QUERY_COMMAND_NOT_FOUND;
-   } else if (code == 0) {
-      code = MONGOC_ERROR_QUERY_FAILURE;
-   }
-
-   bson_set_error (error, domain, code, "%s", msg);
+   _mongoc_cmd_set_error (error, error_api_version, code, msg);
 
    /* there was a command error */
    RETURN (false);
@@ -200,26 +220,18 @@ _mongoc_cmd_check_ok (const bson_t *doc, int32_t error_api_version, bson_error_t
 bool
 _mongoc_cmd_check_ok_no_wce (const bson_t *doc, int32_t error_api_version, bson_error_t *error)
 {
-   mongoc_error_domain_t domain =
-      error_api_version >= MONGOC_ERROR_API_VERSION_2 ? MONGOC_ERROR_SERVER : MONGOC_ERROR_QUERY;
-   uint32_t code;
-   const char *msg = "Unknown command error";
-
    ENTRY;
 
    BSON_ASSERT (doc);
 
+   uint32_t code;
+   const char *msg = "Unknown command error";
+
    if (!_parse_error_reply (doc, true /* check_wce */, &code, &msg)) {
       RETURN (true);
    }
 
-   if (code == MONGOC_ERROR_PROTOCOL_ERROR || code == 13390) {
-      code = MONGOC_ERROR_QUERY_COMMAND_NOT_FOUND;
-   } else if (code == 0) {
-      code = MONGOC_ERROR_QUERY_FAILURE;
-   }
-
-   bson_set_error (error, domain, code, "%s", msg);
+   _mongoc_cmd_set_error (error, error_api_version, code, msg);
 
    /* there was a command error */
    RETURN (false);
@@ -230,18 +242,25 @@ _mongoc_cmd_check_ok_no_wce (const bson_t *doc, int32_t error_api_version, bson_
 static void
 _mongoc_populate_query_error (const bson_t *doc, int32_t error_api_version, bson_error_t *error)
 {
-   mongoc_error_domain_t domain =
-      error_api_version >= MONGOC_ERROR_API_VERSION_2 ? MONGOC_ERROR_SERVER : MONGOC_ERROR_QUERY;
-   uint32_t code = MONGOC_ERROR_QUERY_FAILURE;
-   bson_iter_t iter;
-   const char *msg = "Unknown query failure";
-
    ENTRY;
 
    BSON_ASSERT (doc);
 
+   if (!error) {
+      return;
+   }
+
+   bson_iter_t iter;
+
+   const uint32_t domain = error_api_version >= MONGOC_ERROR_API_VERSION_2 ? MONGOC_ERROR_SERVER : MONGOC_ERROR_QUERY;
+
+   uint32_t code = MONGOC_ERROR_QUERY_FAILURE;
+   uint8_t category = MONGOC_ERROR_CATEGORY;
+   const char *msg = "Unknown query failure";
+
    if (bson_iter_init_find (&iter, doc, "code") && BSON_ITER_HOLDS_NUMBER (&iter)) {
       code = (uint32_t) bson_iter_as_int64 (&iter);
+      category = MONGOC_ERROR_CATEGORY_SERVER;
       BSON_ASSERT (code);
    }
 
@@ -249,7 +268,7 @@ _mongoc_populate_query_error (const bson_t *doc, int32_t error_api_version, bson
       msg = bson_iter_utf8 (&iter, NULL);
    }
 
-   bson_set_error (error, domain, code, "%s", msg);
+   _mongoc_set_error_with_category (error, category, domain, code, "%s", msg);
 
    EXIT;
 }
@@ -265,7 +284,7 @@ mcd_rpc_message_check_ok (mcd_rpc_message *rpc,
    ENTRY;
 
    if (mcd_rpc_header_get_op_code (rpc) != MONGOC_OP_CODE_REPLY) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_PROTOCOL, MONGOC_ERROR_PROTOCOL_INVALID_REPLY, "Received rpc other than OP_REPLY.");
       RETURN (false);
    }
@@ -285,14 +304,14 @@ mcd_rpc_message_check_ok (mcd_rpc_message *rpc,
 
          bson_destroy (&body);
       } else {
-         bson_set_error (error, MONGOC_ERROR_QUERY, MONGOC_ERROR_QUERY_FAILURE, "Unknown query failure.");
+         _mongoc_set_error (error, MONGOC_ERROR_QUERY, MONGOC_ERROR_QUERY_FAILURE, "Unknown query failure.");
       }
 
       RETURN (false);
    }
 
    if (flags & MONGOC_OP_REPLY_RESPONSE_FLAG_CURSOR_NOT_FOUND) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_CURSOR, MONGOC_ERROR_CURSOR_INVALID_CURSOR, "The cursor is invalid or has expired.");
 
       RETURN (false);
