@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,18 +14,23 @@
  * limitations under the License.
  */
 
-#include "common-thread-private.h"
-#include "mongoc-server-monitor-private.h"
+#include <mlib/intencode.h>
+#include <common-thread-private.h>
+#include <mongoc/mongoc-server-monitor-private.h>
 
-#include "mongoc/mcd-rpc.h"
-#include "mongoc/mongoc-client-private.h"
-#include "mongoc/mongoc-error-private.h"
-#include "mongoc/mongoc-handshake-private.h"
-#include "mongoc/mongoc-ssl-private.h"
-#include "mongoc/mongoc-stream-private.h"
-#include "mongoc/mongoc-topology-background-monitoring-private.h"
-#include "mongoc/mongoc-topology-private.h"
-#include "mongoc/mongoc-trace-private.h"
+#include <mongoc/mcd-rpc.h>
+#include <mongoc/mongoc-client-private.h>
+#include <mongoc/mongoc-error-private.h>
+#include <mongoc/mongoc-handshake-private.h>
+#include <mongoc/mongoc-ssl-private.h>
+#include <mongoc/mongoc-stream-private.h>
+#include <mongoc/mongoc-topology-background-monitoring-private.h>
+#include <mongoc/mongoc-topology-private.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-structured-log-private.h>
+#include <common-atomic-private.h>
+
+#include <inttypes.h>
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "monitor"
@@ -80,14 +85,13 @@ struct _mongoc_server_monitor_t {
    mongoc_stream_initiator_t initiator;
    void *initiator_context;
    int32_t request_id;
-   mongoc_apm_callbacks_t apm_callbacks;
-   void *apm_context;
 
    mongoc_stream_t *stream;
    bool more_to_come;
    mongoc_server_description_t *description;
    uint32_t server_id;
    bool is_rtt;
+   mongoc_server_monitoring_mode_t mode;
 };
 
 static BSON_GNUC_PRINTF (3, 4) void _server_monitor_log (mongoc_server_monitor_t *server_monitor,
@@ -120,26 +124,42 @@ static BSON_GNUC_PRINTF (3, 4) void _server_monitor_log (mongoc_server_monitor_t
 
 /* TODO CDRIVER-3710 use MONGOC_LOG_LEVEL_ERROR */
 #define MONITOR_LOG_ERROR(sm, ...) _server_monitor_log (sm, MONGOC_LOG_LEVEL_DEBUG, __VA_ARGS__)
-/* TODO CDRIVER-3710 use MONGOC_LOG_LEVEL_WARNING */
-#define MONITOR_LOG_WARNING(sm, ...) _server_monitor_log (sm, MONGOC_LOG_LEVEL_DEBUG, __VA_ARGS__)
 
 static void
 _server_monitor_heartbeat_started (mongoc_server_monitor_t *server_monitor, bool awaited)
 {
    mongoc_apm_server_heartbeat_started_t event;
-   MONGOC_DEBUG_ASSERT (!mcommon_mutex_is_locked (&server_monitor->topology->apm_mutex));
+   mongoc_log_and_monitor_instance_t *log_and_monitor = &server_monitor->topology->log_and_monitor;
 
-   if (!server_monitor->apm_callbacks.server_heartbeat_started) {
+   {
+      mc_shared_tpld td = mc_tpld_take_ref (BSON_ASSERT_PTR_INLINE (server_monitor)->topology);
+      bson_oid_t topology_id;
+      bson_oid_copy (&td.ptr->topology_id, &topology_id);
+      mc_tpld_drop_ref (&td);
+
+      mongoc_structured_log (
+         log_and_monitor->structured_log,
+         MONGOC_STRUCTURED_LOG_LEVEL_DEBUG,
+         MONGOC_STRUCTURED_LOG_COMPONENT_TOPOLOGY,
+         "Server heartbeat started",
+         oid ("topologyId", &topology_id),
+         server_description (server_monitor->description, SERVER_HOST, SERVER_PORT, SERVER_CONNECTION_ID),
+         boolean ("awaited", awaited));
+   }
+
+   MONGOC_DEBUG_ASSERT (!mcommon_mutex_is_locked (&log_and_monitor->apm_mutex));
+
+   if (!log_and_monitor->apm_callbacks.server_heartbeat_started) {
       return;
    }
 
    event.host = &server_monitor->description->host;
-   event.context = server_monitor->apm_context;
+   event.context = log_and_monitor->apm_context;
    MONITOR_LOG (server_monitor, "%s heartbeat started", awaited ? "awaitable" : "regular");
    event.awaited = awaited;
-   bson_mutex_lock (&server_monitor->topology->apm_mutex);
-   server_monitor->apm_callbacks.server_heartbeat_started (&event);
-   bson_mutex_unlock (&server_monitor->topology->apm_mutex);
+   bson_mutex_lock (&log_and_monitor->apm_mutex);
+   log_and_monitor->apm_callbacks.server_heartbeat_started (&event);
+   bson_mutex_unlock (&log_and_monitor->apm_mutex);
 }
 
 static void
@@ -149,20 +169,39 @@ _server_monitor_heartbeat_succeeded (mongoc_server_monitor_t *server_monitor,
                                      bool awaited)
 {
    mongoc_apm_server_heartbeat_succeeded_t event;
+   mongoc_log_and_monitor_instance_t *log_and_monitor = &server_monitor->topology->log_and_monitor;
 
-   if (!server_monitor->apm_callbacks.server_heartbeat_succeeded) {
+   {
+      mc_shared_tpld td = mc_tpld_take_ref (BSON_ASSERT_PTR_INLINE (server_monitor)->topology);
+      bson_oid_t topology_id;
+      bson_oid_copy (&td.ptr->topology_id, &topology_id);
+      mc_tpld_drop_ref (&td);
+
+      mongoc_structured_log (
+         log_and_monitor->structured_log,
+         MONGOC_STRUCTURED_LOG_LEVEL_DEBUG,
+         MONGOC_STRUCTURED_LOG_COMPONENT_TOPOLOGY,
+         "Server heartbeat succeeded",
+         oid ("topologyId", &topology_id),
+         server_description (server_monitor->description, SERVER_HOST, SERVER_PORT, SERVER_CONNECTION_ID),
+         boolean ("awaited", awaited),
+         monotonic_time_duration (duration_usec),
+         bson_as_json ("reply", reply));
+   }
+
+   if (!log_and_monitor->apm_callbacks.server_heartbeat_succeeded) {
       return;
    }
 
    event.host = &server_monitor->description->host;
-   event.context = server_monitor->apm_context;
+   event.context = log_and_monitor->apm_context;
    event.reply = reply;
    event.duration_usec = duration_usec;
    MONITOR_LOG (server_monitor, "%s heartbeat succeeded", awaited ? "awaitable" : "regular");
    event.awaited = awaited;
-   bson_mutex_lock (&server_monitor->topology->apm_mutex);
-   server_monitor->apm_callbacks.server_heartbeat_succeeded (&event);
-   bson_mutex_unlock (&server_monitor->topology->apm_mutex);
+   bson_mutex_lock (&log_and_monitor->apm_mutex);
+   log_and_monitor->apm_callbacks.server_heartbeat_succeeded (&event);
+   bson_mutex_unlock (&log_and_monitor->apm_mutex);
 }
 
 static void
@@ -172,20 +211,39 @@ _server_monitor_heartbeat_failed (mongoc_server_monitor_t *server_monitor,
                                   bool awaited)
 {
    mongoc_apm_server_heartbeat_failed_t event;
+   mongoc_log_and_monitor_instance_t *log_and_monitor = &server_monitor->topology->log_and_monitor;
 
-   if (!server_monitor->apm_callbacks.server_heartbeat_failed) {
+   {
+      mc_shared_tpld td = mc_tpld_take_ref (BSON_ASSERT_PTR_INLINE (server_monitor)->topology);
+      bson_oid_t topology_id;
+      bson_oid_copy (&td.ptr->topology_id, &topology_id);
+      mc_tpld_drop_ref (&td);
+
+      mongoc_structured_log (
+         log_and_monitor->structured_log,
+         MONGOC_STRUCTURED_LOG_LEVEL_DEBUG,
+         MONGOC_STRUCTURED_LOG_COMPONENT_TOPOLOGY,
+         "Server heartbeat failed",
+         oid ("topologyId", &topology_id),
+         server_description (server_monitor->description, SERVER_HOST, SERVER_PORT, SERVER_CONNECTION_ID),
+         boolean ("awaited", awaited),
+         monotonic_time_duration (duration_usec),
+         error ("failure", error));
+   }
+
+   if (!log_and_monitor->apm_callbacks.server_heartbeat_failed) {
       return;
    }
 
    event.host = &server_monitor->description->host;
-   event.context = server_monitor->apm_context;
+   event.context = log_and_monitor->apm_context;
    event.error = error;
    event.duration_usec = duration_usec;
    MONITOR_LOG (server_monitor, "%s heartbeat failed", awaited ? "awaitable" : "regular");
    event.awaited = awaited;
-   bson_mutex_lock (&server_monitor->topology->apm_mutex);
-   server_monitor->apm_callbacks.server_heartbeat_failed (&event);
-   bson_mutex_unlock (&server_monitor->topology->apm_mutex);
+   bson_mutex_lock (&log_and_monitor->apm_mutex);
+   log_and_monitor->apm_callbacks.server_heartbeat_failed (&event);
+   bson_mutex_unlock (&log_and_monitor->apm_mutex);
 }
 
 static void
@@ -200,13 +258,6 @@ _server_monitor_append_cluster_time (mongoc_server_monitor_t *server_monitor, bs
    mc_tpld_drop_ref (&td);
 }
 
-static int32_t
-_int32_from_le (const void *data)
-{
-   BSON_ASSERT_PARAM (data);
-   return bson_iter_int32_unsafe (&(bson_iter_t){.raw = data});
-}
-
 static bool
 _server_monitor_send_and_recv_hello_opmsg (mongoc_server_monitor_t *server_monitor,
                                            const bson_t *cmd,
@@ -258,16 +309,16 @@ _server_monitor_send_and_recv_hello_opmsg (mongoc_server_monitor_t *server_monit
       goto fail;
    }
 
-   const int32_t message_length = _int32_from_le (buffer.data);
+   const int32_t message_length = mlib_read_i32le (buffer.data);
 
    // msgHeader consists of four int32 fields.
    const int32_t message_header_length = 4u * sizeof (int32_t);
 
    if (message_length < message_header_length) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: message length");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: message length");
       goto fail;
    }
 
@@ -280,29 +331,29 @@ _server_monitor_send_and_recv_hello_opmsg (mongoc_server_monitor_t *server_monit
 
    mcd_rpc_message_reset (rpc);
    if (!mcd_rpc_message_from_data_in_place (rpc, buffer.data, buffer.len, NULL)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: malformed message");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: malformed message");
       goto fail;
    }
 
    mcd_rpc_message_ingress (rpc);
 
    if (!mcd_rpc_message_decompress_if_necessary (rpc, &decompressed_data, &decompressed_data_len)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: decompression failure");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: decompression failure");
       goto fail;
    }
 
    bson_t body;
    if (!mcd_rpc_message_get_body (rpc, &body)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: malformed body");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: malformed body");
       goto fail;
    }
 
@@ -371,16 +422,16 @@ _server_monitor_send_and_recv_opquery (mongoc_server_monitor_t *server_monitor,
       goto fail;
    }
 
-   const int32_t message_length = _int32_from_le (buffer.data);
+   const int32_t message_length = mlib_read_i32le (buffer.data);
 
    // msgHeader consists of four int32 fields.
    const int32_t message_header_length = 4u * sizeof (int32_t);
 
    if (message_length < message_header_length) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: message length");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: message length");
       goto fail;
    }
 
@@ -393,29 +444,29 @@ _server_monitor_send_and_recv_opquery (mongoc_server_monitor_t *server_monitor,
 
    mcd_rpc_message_reset (rpc);
    if (!mcd_rpc_message_from_data_in_place (rpc, buffer.data, buffer.len, NULL)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: malformed message");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: malformed message");
       goto fail;
    }
 
    mcd_rpc_message_ingress (rpc);
 
    if (!mcd_rpc_message_decompress_if_necessary (rpc, &decompressed_data, &decompressed_data_len)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: decompression failure");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: decompression failure");
       goto fail;
    }
 
    bson_t body;
    if (!mcd_rpc_message_get_body (rpc, &body)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid reply from server: malformed body");
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "invalid reply from server: malformed body");
       goto fail;
    }
 
@@ -540,7 +591,7 @@ _server_monitor_poll_with_interrupt (mongoc_server_monitor_t *server_monitor,
       ssize_t ret;
       mongoc_stream_poll_t poller[1];
 
-      MONITOR_LOG (server_monitor, "_server_monitor_poll_with_interrupt expires in: %" PRIu64 "ms", timeleft_ms);
+      MONITOR_LOG (server_monitor, "_server_monitor_poll_with_interrupt expires in: %" PRId64 "ms", timeleft_ms);
       poller[0].stream = server_monitor->stream;
       poller[0].events = POLLIN; /* POLLERR and POLLHUP are added in mongoc_socket_poll. */
       poller[0].revents = 0;
@@ -549,12 +600,12 @@ _server_monitor_poll_with_interrupt (mongoc_server_monitor_t *server_monitor,
       ret = mongoc_stream_poll (poller, 1, (int32_t) BSON_MIN (timeleft_ms, monitor_tick_ms));
       if (ret == -1) {
          MONITOR_LOG (server_monitor, "mongoc_stream_poll error");
-         bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "poll error");
+         _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "poll error");
          return false;
       }
 
       if (poller[0].revents & (POLLERR | POLLHUP)) {
-         bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "connection closed while polling");
+         _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "connection closed while polling");
          return false;
       }
 
@@ -574,7 +625,7 @@ _server_monitor_poll_with_interrupt (mongoc_server_monitor_t *server_monitor,
          return true;
       }
    }
-   bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "connection timeout while polling");
+   _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "connection timeout while polling");
    return false;
 }
 
@@ -590,7 +641,7 @@ _get_timeout_ms (int64_t expire_at_ms, bson_error_t *error)
 
    timeout_ms = expire_at_ms - _now_ms ();
    if (timeout_ms <= 0) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "connection timed out reading message length");
       return 0;
    }
@@ -640,18 +691,18 @@ _server_monitor_awaitable_hello_recv (mongoc_server_monitor_t *server_monitor,
       GOTO (fail);
    }
 
-   const int32_t message_length = _int32_from_le (buffer.data);
+   const int32_t message_length = mlib_read_i32le (buffer.data);
 
    // msgHeader consists of four int32 fields.
    const int32_t message_header_length = 4u * sizeof (int32_t);
 
    if ((message_length < message_header_length) || (message_length > server_monitor->description->max_msg_size)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_PROTOCOL,
-                      MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "message size %" PRId32 " is not within expected range 16-%" PRId32 " bytes",
-                      message_length,
-                      server_monitor->description->max_msg_size);
+      _mongoc_set_error (error,
+                         MONGOC_ERROR_PROTOCOL,
+                         MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
+                         "message size %" PRId32 " is not within expected range 16-%" PRId32 " bytes",
+                         message_length,
+                         server_monitor->description->max_msg_size);
       GOTO (fail);
    }
 
@@ -668,7 +719,7 @@ _server_monitor_awaitable_hello_recv (mongoc_server_monitor_t *server_monitor,
    }
 
    if (!mcd_rpc_message_from_data_in_place (rpc, buffer.data, buffer.len, NULL)) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_PROTOCOL, MONGOC_ERROR_PROTOCOL_INVALID_REPLY, "malformed message from server");
       GOTO (fail);
    }
@@ -676,13 +727,13 @@ _server_monitor_awaitable_hello_recv (mongoc_server_monitor_t *server_monitor,
    mcd_rpc_message_ingress (rpc);
 
    if (!mcd_rpc_message_decompress_if_necessary (rpc, &decompressed_data, &decompressed_data_len)) {
-      bson_set_error (error, MONGOC_ERROR_PROTOCOL, MONGOC_ERROR_PROTOCOL_INVALID_REPLY, "decompression failure");
+      _mongoc_set_error (error, MONGOC_ERROR_PROTOCOL, MONGOC_ERROR_PROTOCOL_INVALID_REPLY, "decompression failure");
       GOTO (fail);
    }
 
    bson_t body;
    if (!mcd_rpc_message_get_body (rpc, &body)) {
-      bson_set_error (
+      _mongoc_set_error (
          error, MONGOC_ERROR_PROTOCOL, MONGOC_ERROR_PROTOCOL_INVALID_REPLY, "malformed BSON payload from server");
       GOTO (fail);
    }
@@ -771,7 +822,7 @@ _update_topology_description (mongoc_server_monitor_t *server_monitor, mongoc_se
       _mongoc_topology_update_cluster_time (topology, hello_response);
    }
 
-   if (bson_atomic_int_fetch (&topology->scanner_state, bson_memory_order_relaxed) ==
+   if (mcommon_atomic_int_fetch (&topology->scanner_state, mcommon_memory_order_relaxed) ==
        MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN) {
       return;
    }
@@ -780,8 +831,12 @@ _update_topology_description (mongoc_server_monitor_t *server_monitor, mongoc_se
    bson_mutex_lock (&server_monitor->shared.mutex);
    server_monitor->shared.scan_requested = false;
    bson_mutex_unlock (&server_monitor->shared.mutex);
-   mongoc_topology_description_handle_hello (
-      tdmod.new_td, server_monitor->server_id, hello_response, description->round_trip_time_msec, &description->error);
+   mongoc_topology_description_handle_hello (tdmod.new_td,
+                                             &topology->log_and_monitor,
+                                             server_monitor->server_id,
+                                             hello_response,
+                                             description->round_trip_time_msec,
+                                             &description->error);
    /* Reconcile server monitors. */
    _mongoc_topology_background_monitoring_reconcile (topology, tdmod.new_td);
    /* Wake threads performing server selection. */
@@ -789,6 +844,24 @@ _update_topology_description (mongoc_server_monitor_t *server_monitor, mongoc_se
    mc_tpld_modify_commit (tdmod);
 }
 
+/* Get the mode enum based on the uri
+ *
+ * Called during server monitor creation
+ */
+static mongoc_server_monitoring_mode_t
+_server_monitor_get_mode_enum (mongoc_server_monitor_t *server_monitor)
+{
+   const char *mode_str = mongoc_uri_get_server_monitoring_mode (server_monitor->uri);
+
+   if (strcmp (mode_str, "poll") == 0) {
+      return MONGOC_SERVER_MONITORING_POLL;
+   } else if (strcmp (mode_str, "stream") == 0) {
+      return MONGOC_SERVER_MONITORING_STREAM;
+   } else {
+      return MONGOC_SERVER_MONITORING_AUTO;
+   }
+}
+
 /* Create a new server monitor.
  *
  * Called during reconcile.
@@ -816,10 +889,9 @@ mongoc_server_monitor_new (mongoc_topology_t *topology,
       _mongoc_ssl_opts_copy_to (topology->scanner->ssl_opts, server_monitor->ssl_opts, true);
    }
 #endif
-   memcpy (&server_monitor->apm_callbacks, &td->apm_callbacks, sizeof (mongoc_apm_callbacks_t));
-   server_monitor->apm_context = td->apm_context;
    server_monitor->initiator = topology->scanner->initiator;
    server_monitor->initiator_context = topology->scanner->initiator_context;
+   server_monitor->mode = _server_monitor_get_mode_enum (server_monitor);
    mongoc_cond_init (&server_monitor->shared.cond);
    bson_mutex_init (&server_monitor->shared.mutex);
    return server_monitor;
@@ -855,12 +927,23 @@ _server_monitor_setup_connection (mongoc_server_monitor_t *server_monitor,
          server_monitor->uri, &server_monitor->description->host, server_monitor->initiator_context, error);
    } else {
       void *ssl_opts_void = NULL;
+      void *openssl_ctx_void = NULL;
 
 #ifdef MONGOC_ENABLE_SSL
       ssl_opts_void = server_monitor->ssl_opts;
 #endif
-      server_monitor->stream = mongoc_client_connect (
-         false, ssl_opts_void != NULL, ssl_opts_void, server_monitor->uri, &server_monitor->description->host, error);
+
+#if defined(MONGOC_ENABLE_SSL_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10100000L
+      openssl_ctx_void = server_monitor->topology->scanner->openssl_ctx;
+#endif
+
+      server_monitor->stream = mongoc_client_connect (false,
+                                                      ssl_opts_void != NULL,
+                                                      ssl_opts_void,
+                                                      server_monitor->uri,
+                                                      &server_monitor->description->host,
+                                                      openssl_ctx_void,
+                                                      error);
    }
 
    if (!server_monitor->stream) {
@@ -938,11 +1021,15 @@ _server_monitor_check_server (mongoc_server_monitor_t *server_monitor,
       GOTO (exit);
    }
 
-   if (!bson_empty (&previous_description->topology_version) &&
-       _mongoc_handshake_get ()->env == MONGOC_HANDSHAKE_ENV_NONE) {
+   if (server_monitor->mode != MONGOC_SERVER_MONITORING_POLL && !bson_empty (&previous_description->topology_version) &&
+       (_mongoc_handshake_get ()->env == MONGOC_HANDSHAKE_ENV_NONE ||
+        server_monitor->mode == MONGOC_SERVER_MONITORING_STREAM)) {
       // Use stream monitoring if:
+      // - serverMonitoringMode != "poll"
       // - Server supports stream monitoring (indicated by `topologyVersion`).
-      // - Application is not in an FaaS environment (e.g. AWS Lambda).
+      // - ONE OF:
+      //    - Application is not in an FaaS environment (e.g. AWS Lambda).
+      //    - serverMonitoringMode == "stream"
       awaited = true;
       _server_monitor_heartbeat_started (server_monitor, awaited);
       MONITOR_LOG (server_monitor, "awaitable hello");
