@@ -28,7 +28,7 @@
 #include <Mstcpip.h>
 #include <process.h>
 #endif
-#include <common-cmp-private.h>
+#include <mlib/cmp.h>
 
 #include <inttypes.h>
 
@@ -1192,7 +1192,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
 
    for (size_t i = 0u; i < iovcnt; i++) {
 #ifdef _WIN32
-      BSON_ASSERT (mcommon_in_range_unsigned (int, iov[i].iov_len));
+      BSON_ASSERT (mlib_in_range (int, iov[i].iov_len));
       const int wrote = send (sock->sd, iov[i].iov_base, (int) iov[i].iov_len, 0);
       if (wrote == SOCKET_ERROR) {
 #else
@@ -1209,7 +1209,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
 
       ret += wrote;
 
-      if (mcommon_cmp_not_equal_su (wrote, iov[i].iov_len)) {
+      if (mlib_cmp (wrote, !=, iov[i].iov_len)) {
          RETURN (ret);
       }
    }
@@ -1259,7 +1259,7 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
    DUMP_IOVEC (sendbuf, iov, iovcnt);
 
 #ifdef _WIN32
-   BSON_ASSERT (mcommon_in_range_unsigned (unsigned_long, iovcnt));
+   BSON_ASSERT (mlib_in_range (unsigned long, iovcnt));
    ret = WSASend (sock->sd, (LPWSABUF) iov, (DWORD) iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
    TRACE ("WSASend sent: %lu (out of: %zu), ret: %d", dwNumberofBytesSent, iov->iov_len, ret);
 #else
