@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-#include "mongoc-config.h"
+#include <mongoc/mongoc-config.h>
 
 #ifdef MONGOC_ENABLE_SSL
 
@@ -22,26 +22,25 @@
 #include <string.h>
 #include <bson/bson.h>
 
-#include "mongoc-log.h"
-#include "mongoc-trace-private.h"
-#include "mongoc-error.h"
+#include <mongoc/mongoc-log.h>
+#include <mongoc/mongoc-trace-private.h>
+#include <mongoc/mongoc-error-private.h>
 
-#include "mongoc-stream-tls-private.h"
-#include "mongoc-stream-private.h"
+#include <mongoc/mongoc-stream-tls-private.h>
+#include <mongoc/mongoc-stream-private.h>
 #if defined(MONGOC_ENABLE_SSL_OPENSSL)
-#include "mongoc-stream-tls-openssl.h"
-#include "mongoc-openssl-private.h"
-#elif defined(MONGOC_ENABLE_SSL_LIBRESSL)
-#include "mongoc-libressl-private.h"
-#include "mongoc-stream-tls-libressl.h"
+#include <mongoc/mongoc-stream-tls-openssl.h>
+#include <mongoc/mongoc-openssl-private.h>
 #elif defined(MONGOC_ENABLE_SSL_SECURE_TRANSPORT)
-#include "mongoc-secure-transport-private.h"
-#include "mongoc-stream-tls-secure-transport.h"
+#include <mongoc/mongoc-secure-transport-private.h>
+#include <mongoc/mongoc-stream-tls-secure-transport.h>
 #elif defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL)
-#include "mongoc-secure-channel-private.h"
-#include "mongoc-stream-tls-secure-channel.h"
+#include <mongoc/mongoc-secure-channel-private.h>
+#include <mongoc/mongoc-stream-tls-secure-channel.h>
 #endif
-#include "mongoc-stream-tls.h"
+#include <mongoc/mongoc-stream-tls.h>
+#include <common-macros-private.h> // BEGIN_IGNORE_DEPRECATIONS
+#include <mlib/cmp.h>
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "stream-tls"
@@ -104,11 +103,11 @@ mongoc_stream_tls_handshake_block (mongoc_stream_t *stream, const char *host, in
             const int64_t now = bson_get_monotonic_time ();
             const int64_t remaining = expire - now;
             if (remaining < 0) {
-               bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "TLS handshake timed out.");
+               _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "TLS handshake timed out.");
                return false;
             } else {
                const int64_t msec = remaining / 1000;
-               BSON_ASSERT (bson_in_range_signed (int32_t, msec));
+               BSON_ASSERT (mlib_in_range (int32_t, msec));
                timeout_msec = (int32_t) msec;
             }
          }
@@ -117,46 +116,10 @@ mongoc_stream_tls_handshake_block (mongoc_stream_t *stream, const char *host, in
    } while (events && ret > 0);
 
    if (error && !error->code) {
-      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "TLS handshake failed.");
+      _mongoc_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "TLS handshake failed.");
    }
    return false;
 }
-/**
- * Deprecated. Was never supposed to be part of the public API.
- * See mongoc_stream_tls_handshake.
- */
-bool
-mongoc_stream_tls_do_handshake (mongoc_stream_t *stream, int32_t timeout_msec)
-{
-   mongoc_stream_tls_t *stream_tls = (mongoc_stream_tls_t *) mongoc_stream_get_tls_stream (stream);
-
-   BSON_UNUSED (timeout_msec);
-
-   BSON_ASSERT (stream_tls);
-
-   MONGOC_ERROR ("This function doesn't do anything. Please call "
-                 "mongoc_stream_tls_handshake()");
-   return false;
-}
-
-
-/**
- * Deprecated. Was never supposed to be part of the public API.
- * See mongoc_stream_tls_handshake.
- */
-bool
-mongoc_stream_tls_check_cert (mongoc_stream_t *stream, const char *host)
-{
-   mongoc_stream_tls_t *stream_tls = (mongoc_stream_tls_t *) mongoc_stream_get_tls_stream (stream);
-
-   BSON_UNUSED (host);
-
-   BSON_ASSERT (stream_tls);
-
-   MONGOC_ERROR ("This function doesn't do anything. Please call "
-                 "mongoc_stream_tls_handshake()");
-   return false;
-}
 
 
 /*
@@ -205,8 +168,6 @@ mongoc_stream_tls_new_with_hostname (mongoc_stream_t *base_stream, const char *h
 
 #if defined(MONGOC_ENABLE_SSL_OPENSSL)
    return mongoc_stream_tls_openssl_new (base_stream, host, opt, client);
-#elif defined(MONGOC_ENABLE_SSL_LIBRESSL)
-   return mongoc_stream_tls_libressl_new (base_stream, host, opt, client);
 #elif defined(MONGOC_ENABLE_SSL_SECURE_TRANSPORT)
    return mongoc_stream_tls_secure_transport_new (base_stream, host, opt, client);
 #elif defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL)
@@ -216,10 +177,54 @@ mongoc_stream_tls_new_with_hostname (mongoc_stream_t *base_stream, const char *h
 #endif
 }
 
+#if defined(MONGOC_ENABLE_SSL_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10100000L
+/*
+ *--------------------------------------------------------------------------
+ *
+ * mongoc_stream_tls_new_with_hostname_and_openssl_context --
+ *
+ *       Creates a new mongoc_stream_tls_t to communicate with a remote
+ *       server using a TLS stream, using an existing OpenSSL context.
+ *
+ *       @ssl_ctx is the global OpenSSL context for the mongoc_client_t
+ *       associated with this function call.
+ *
+ *       @host the hostname we are connected to and to verify the
+ *       server certificate against
+ *
+ *       @base_stream should be a stream that will become owned by the
+ *       resulting tls stream. It will be used for raw I/O.
+ *
+ * Returns:
+ *       NULL on failure, otherwise a mongoc_stream_t.
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+
 mongoc_stream_t *
-mongoc_stream_tls_new (mongoc_stream_t *base_stream, mongoc_ssl_opt_t *opt, int client)
+mongoc_stream_tls_new_with_hostname_and_openssl_context (
+   mongoc_stream_t *base_stream, const char *host, mongoc_ssl_opt_t *opt, int client, SSL_CTX *ssl_ctx)
 {
-   return mongoc_stream_tls_new_with_hostname (base_stream, NULL, opt, client);
+   BSON_ASSERT (base_stream);
+
+   /* !client is only used for testing,
+    * when the streams are pretending to be the server */
+   if (!client || opt->weak_cert_validation) {
+      opt->allow_invalid_hostname = true;
+   }
+
+#ifndef _WIN32
+   /* Silly check for Unix Domain Sockets */
+   if (!host || (host[0] == '/' && !access (host, F_OK))) {
+      opt->allow_invalid_hostname = true;
+   }
+#endif
+
+   return mongoc_stream_tls_openssl_new_with_context (base_stream, host, opt, client, ssl_ctx);
 }
+#endif
 
 #endif
