@@ -14,13 +14,13 @@
  * limitations under the License.
  */
 
-#include "mongoc-prelude.h"
+#include <mongoc/mongoc-prelude.h>
 
 #ifndef MONGOC_UTIL_PRIVATE_H
 #define MONGOC_UTIL_PRIVATE_H
 
 #include <bson/bson.h>
-#include "mongoc.h"
+#include <mongoc/mongoc.h>
 
 #ifdef BSON_HAVE_STRINGS_H
 #include <strings.h>
@@ -85,13 +85,6 @@ _mongoc_bson_type_to_str (bson_type_t t);
 const char *
 _mongoc_wire_version_to_server_version (int32_t version);
 
-bool
-_mongoc_get_server_id_from_opts (const bson_t *opts,
-                                 mongoc_error_domain_t domain,
-                                 mongoc_error_code_t code,
-                                 uint32_t *server_id,
-                                 bson_error_t *error);
-
 bool
 _mongoc_validate_new_document (const bson_t *insert, bson_validate_flags_t vflags, bson_error_t *error);
 
@@ -245,6 +238,11 @@ hex_to_bin (const char *hex, uint32_t *len);
 char *
 bin_to_hex (const uint8_t *bin, uint32_t len);
 
+typedef struct {
+   bool set;
+   uint64_t value;
+} mcd_optional_u64_t;
+
 BSON_END_DECLS
 
 #endif /* MONGOC_UTIL_PRIVATE_H */
