@@ -30,7 +30,8 @@
 #include "../TestSuite.h"
 #include <common-string-private.h>
 #include <common-json-private.h>
-#include <common-cmp-private.h>
+#include <mlib/cmp.h>
+#include <mlib/intencode.h>
 
 #ifdef BSON_HAVE_STRINGS_H
 #include <strings.h>
@@ -356,7 +357,7 @@ mock_server_run (mock_server_t *server)
    // socklen_t: an unsigned opaque integral type of length of at least 32 bits.
    // To forestall portability problems, it is recommended that applications
    // should not use values larger than 2^32 - 1.
-   BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, bind_addr_len));
+   BSON_ASSERT (mlib_in_range (uint32_t, bind_addr_len));
 
    if (-1 == mongoc_socket_bind (ssock, (struct sockaddr *) bind_addr, (uint32_t) bind_addr_len)) {
       perror ("Failed to bind socket");
@@ -1803,7 +1804,6 @@ static BSON_THREAD_FUN (worker_thread, data)
    mongoc_buffer_t buffer;
    bool handled;
    bson_error_t error;
-   int32_t msg_len;
    sync_queue_t *requests;
    sync_queue_t *replies;
    request_t *request;
@@ -1844,9 +1844,7 @@ again:
    /* loop, checking for requests to receive or replies to send */
    if (_mongoc_buffer_fill (&buffer, client_stream, 4, 10, &error) > 0) {
       BSON_ASSERT (buffer.len >= 4);
-
-      memcpy (&msg_len, buffer.data, 4);
-      msg_len = BSON_UINT32_FROM_LE (msg_len);
+      const int32_t msg_len = mlib_read_u32le (buffer.data);
 
       if (msg_len < 16) {
          MONGOC_WARNING ("No data");
@@ -2065,7 +2063,7 @@ _mock_server_reply_with_stream (mock_server_t *server, reply_t *reply, mongoc_st
 
    const ssize_t n_written = mongoc_stream_writev (client, iov, iovcnt, -1);
 
-   BSON_ASSERT (mcommon_cmp_equal_su (n_written, expected));
+   BSON_ASSERT (mlib_cmp (n_written, ==, expected));
 
    bson_free (iov);
    mcd_rpc_message_destroy (rpc);
