@@ -319,8 +319,8 @@ _testcase_run (he_testcase_t *testcase)
    duration_ms = (bson_get_monotonic_time () - testcase->state.start) / (1000);
 
    {
-      bool within_expected_duration =
-         duration_ms >= expected->duration_min_ms && duration_ms < expected->duration_max_ms;
+      bool within_expected_duration = mlib_cmp (duration_ms, >=, expected->duration_min_ms) //
+                                      && mlib_cmp (duration_ms, <, expected->duration_max_ms);
       if (!within_expected_duration) {
          /* this is a timing failure, this may have been a fluke, retry once. */
          ASSERT_WITH_MSG (!testcase->state.last_duration,
