@@ -4,6 +4,7 @@
 #include <mongoc/mongoc-change-stream-private.h>
 #include <mongoc/mongoc-collection-private.h>
 #include <mongoc/utlist.h>
+#include <mlib/loop.h>
 #include "TestSuite.h"
 #include "test-conveniences.h"
 #include "test-libmongoc.h"
@@ -14,9 +15,6 @@
 
 #include <inttypes.h>
 
-#undef MONGOC_LOG_DOMAIN
-#define MONGOC_LOG_DOMAIN "session-test"
-
 static void
 test_session_opts_clone (void)
 {
@@ -787,7 +785,6 @@ _test_end_sessions_many (bool pooled)
 {
    endsessions_test_t test;
    mongoc_client_t *client;
-   int i;
    mongoc_client_session_t *sessions[10001];
    bson_error_t error;
    bson_t ended_lsids;
@@ -801,14 +798,13 @@ _test_end_sessions_many (bool pooled)
    /*
     * create and destroy 10,001 sessions
     */
-   for (i = 0; i < sizeof sessions / sizeof (mongoc_client_session_t *); i++) {
-      sessions[i] = mongoc_client_start_session (client, NULL, &error);
-      ASSERT_OR_PRINT (sessions[i], error);
-      send_ping (client, sessions[i]);
+   mlib_foreach_arr (mongoc_client_session_t *, session, sessions) {
+      *session = mongoc_client_start_session (client, NULL, &error);
+      ASSERT_OR_PRINT (*session, error);
+      send_ping (client, *session);
    }
-
-   for (i = 0; i < sizeof sessions / sizeof (mongoc_client_session_t *); i++) {
-      mongoc_client_session_destroy (sessions[i]);
+   mlib_foreach_arr (mongoc_client_session_t *, session, sessions) {
+      mongoc_client_session_destroy (*session);
    }
 
    endsessions_test_destroy_client (&test);
@@ -1604,15 +1600,6 @@ run_session_test_bulk_operation (void *ctx)
 }
 
 
-static void
-run_count_test (void *ctx)
-{
-   /* CDRIVER-3612: mongoc_collection_estimated_document_count does not support
-    * explicit sessions */
-   _test_implicit_session_lsid (((session_test_helper_t *) ctx)->test_fn);
-}
-
-
 static void
 insert_10_docs (session_test_t *test)
 {
@@ -1685,14 +1672,6 @@ test_db_cmd (session_test_t *test)
 }
 
 
-static void
-test_count (session_test_t *test)
-{
-   test->succeeded = (-1 != mongoc_collection_count_with_opts (
-                               test->collection, MONGOC_QUERY_NONE, NULL, 0, 0, &test->opts, NULL, &test->error));
-}
-
-
 static void
 test_cursor (session_test_t *test)
 {
@@ -1742,10 +1721,10 @@ test_drop_index (session_test_t *test)
 static void
 test_create_index (session_test_t *test)
 {
-   BEGIN_IGNORE_DEPRECATIONS
-   test->succeeded = mongoc_collection_create_index_with_opts (
-      test->collection, tmp_bson ("{'a': 1}"), NULL, &test->opts, NULL, &test->error);
-   END_IGNORE_DEPRECATIONS
+   mongoc_index_model_t *im = mongoc_index_model_new (tmp_bson ("{'a': 1}"), NULL);
+   test->succeeded =
+      mongoc_collection_create_indexes_with_opts (test->collection, &im, 1, &test->opts, NULL, &test->error);
+   mongoc_index_model_destroy (im);
 }
 
 static void
@@ -2751,17 +2730,6 @@ test_session_install (TestSuite *suite)
    add_session_test (suite, "/Session/write_cmd", test_write_cmd, false);
    add_session_test (suite, "/Session/read_write_cmd", test_read_write_cmd, true);
    add_session_test (suite, "/Session/db_cmd", test_db_cmd, false);
-   {
-      session_test_helper_t *const helper = bson_malloc (sizeof (*helper));
-      *helper = (session_test_helper_t){.test_fn = test_count};
-      TestSuite_AddFull (suite,
-                         "/Session/count",
-                         run_count_test,
-                         bson_free,
-                         helper,
-                         test_framework_skip_if_no_cluster_time,
-                         test_framework_skip_if_no_crypto);
-   }
    add_session_test (suite, "/Session/cursor", test_cursor, true);
    add_session_test (suite, "/Session/drop", test_drop, false);
    add_session_test (suite, "/Session/drop_index", test_drop_index, false);
