@@ -549,39 +549,6 @@ test_cluster_time_command_simple_pooled (void)
 }
 
 
-/* test the deprecated mongoc_client_command function with $clusterTime */
-static bool
-client_command (mongoc_client_t *client, bson_error_t *error)
-{
-   mongoc_cursor_t *cursor;
-   const bson_t *doc;
-   bool r;
-
-   ASSERT (client);
-
-   cursor = mongoc_client_command (client, "test", MONGOC_QUERY_NONE, 0, 0, 0, tmp_bson ("{'ping': 1}"), NULL, NULL);
-
-   mongoc_cursor_next (cursor, &doc);
-   r = !mongoc_cursor_error (cursor, error);
-   mongoc_cursor_destroy (cursor);
-   return r;
-}
-
-
-static void
-test_cluster_time_command_single (void)
-{
-   _test_cluster_time (false, client_command);
-}
-
-
-static void
-test_cluster_time_command_pooled (void)
-{
-   _test_cluster_time (true, client_command);
-}
-
-
 /* test modern mongoc_client_read_command_with_opts with $clusterTime */
 static bool
 client_command_with_opts (mongoc_client_t *client, bson_error_t *error)
@@ -1107,204 +1074,6 @@ test_not_primary_auth_pooled (void)
 }
 
 
-typedef struct {
-   const char *name;
-   const char *q;
-   const char *e;
-   bool secondary;
-   bool cluster_time;
-} dollar_query_test_t;
-
-
-static bool
-auto_hello_callback (request_t *request, void *data, bson_t *hello_response)
-{
-   dollar_query_test_t *test;
-   bson_t cluster_time;
-
-   BSON_UNUSED (request);
-
-   test = (dollar_query_test_t *) data;
-
-   bson_init (hello_response);
-   BSON_APPEND_INT32 (hello_response, "ok", 1);
-   BSON_APPEND_BOOL (hello_response, "isWritablePrimary", !test->secondary);
-   BSON_APPEND_BOOL (hello_response, "secondary", test->secondary);
-   BSON_APPEND_INT32 (hello_response, "minWireVersion", WIRE_VERSION_MIN);
-   BSON_APPEND_INT32 (hello_response, "maxWireVersion", WIRE_VERSION_MAX);
-   BSON_APPEND_UTF8 (hello_response, "setName", "rs");
-
-   if (test->cluster_time) {
-      BSON_APPEND_DOCUMENT_BEGIN (hello_response, "$clusterTime", &cluster_time);
-      BSON_APPEND_TIMESTAMP (&cluster_time, "clusterTime", 1, 1);
-      bson_append_document_end (hello_response, &cluster_time);
-   }
-
-   return true;
-}
-
-
-static void
-_test_dollar_query (void *ctx)
-{
-   dollar_query_test_t *test;
-   mock_server_t *server;
-   mongoc_client_t *client;
-   mongoc_collection_t *collection;
-   mongoc_read_prefs_t *read_prefs;
-   mongoc_cursor_t *cursor;
-   future_t *future;
-   request_t *request;
-   const bson_t *doc;
-   bson_error_t error;
-
-   test = (dollar_query_test_t *) ctx;
-
-   server = mock_server_new ();
-   mock_server_auto_hello_callback (server, auto_hello_callback, test, NULL);
-   mock_server_run (server);
-
-   client = test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
-   collection = mongoc_client_get_collection (client, "db", "collection");
-   if (test->secondary) {
-      read_prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
-   } else {
-      read_prefs = NULL;
-   }
-
-   cursor = mongoc_collection_find (collection, MONGOC_QUERY_NONE, 0, 0, 0, tmp_bson (test->q), NULL, read_prefs);
-
-   future = future_cursor_next (cursor, &doc);
-   request = mock_server_receives_msg (server, 0, tmp_bson (test->e));
-   reply_to_op_msg_request (request,
-                            MONGOC_MSG_NONE,
-                            tmp_bson ("{'ok': 1,"
-                                      " 'cursor': {"
-                                      "    'id': {'$numberLong': '0'},"
-                                      "    'ns': 'db.collection',"
-                                      "    'firstBatch': []}}"));
-
-   BSON_ASSERT (!future_get_bool (future));
-   ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
-
-   future_destroy (future);
-   request_destroy (request);
-   mongoc_read_prefs_destroy (read_prefs);
-   mongoc_cursor_destroy (cursor);
-
-   mongoc_collection_destroy (collection);
-   mongoc_client_destroy (client);
-   mock_server_destroy (server);
-}
-
-
-dollar_query_test_t tests[] = {{"/Cluster/cluster_time/query/",
-                                "{'a': 1}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': false}"
-                                "}",
-                                false,
-                                false},
-                               {"/Cluster/cluster_time/query/cluster_time",
-                                "{'a': 1}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': true}"
-                                "}",
-                                false,
-                                true},
-                               {"/Cluster/cluster_time/query/secondary",
-                                "{'a': 1}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1}, "
-                                "   '$clusterTime': {'$exists': false},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                false},
-                               {"/Cluster/cluster_time/query/cluster_time_secondary",
-                                "{'a': 1}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1}, "
-                                "   '$clusterTime': {'$exists': true},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                true},
-                               {"/Cluster/cluster_time/dollar_query/from_user",
-                                "{'$query': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': false}"
-                                "}",
-                                false,
-                                false},
-                               {"/Cluster/cluster_time/dollar_query/from_user/cluster_time",
-                                "{'$query': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': true}"
-                                "}",
-                                false,
-                                true},
-                               {"/Cluster/cluster_time/dollar_query/from_user/secondary",
-                                "{'$query': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': false},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                false},
-                               {"/Cluster/cluster_time/dollar_query/from_user/cluster_time_secondary",
-                                "{'$query': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   '$clusterTime': {'$exists': true},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                true},
-                               {"/Cluster/cluster_time/dollar_orderby",
-                                "{'$query': {'a': 1}, '$orderby': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   'sort': {'a': 1}"
-                                "}",
-                                false,
-                                false},
-                               {"/Cluster/cluster_time/dollar_orderby/secondary",
-                                "{'$query': {'a': 1}, '$orderby': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   'sort': {'a': 1},"
-                                "   '$clusterTime': {'$exists': false},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                false},
-                               {"/Cluster/cluster_time/dollar_orderby/cluster_time",
-                                "{'$query': {'a': 1}, '$orderby': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   'sort': {'a': 1},"
-                                "   '$clusterTime': {'$exists': true}"
-                                "}",
-                                false,
-                                true},
-                               {"/Cluster/cluster_time/dollar_orderby/cluster_time_secondary",
-                                "{'$query': {'a': 1}, '$orderby': {'a': 1}}",
-                                "{"
-                                "   'find': 'collection', 'filter': {'a': 1},"
-                                "   'sort': {'a': 1},"
-                                "   '$clusterTime': {'$exists': true},"
-                                "   '$readPreference': {'mode': 'secondary'}"
-                                "}",
-                                true,
-                                true},
-                               {NULL}};
-
 static void
 _test_cluster_hello_fails (bool hangup)
 {
@@ -1553,7 +1322,7 @@ test_hello_on_unknown (void)
 
    client = mongoc_client_pool_pop (pool);
 
-   mongoc_client_set_stream_initiator (client, _initiator_fn, pool);
+   _mongoc_client_set_stream_initiator_single_or_pooled (client, _initiator_fn, pool);
 
    /* The other client marked the server as unknown after this client selected
     * the server and created a stream, but *before* constructing the initial
@@ -1775,14 +1544,6 @@ test_cluster_stream_invalidation_pooled (void)
 void
 test_cluster_install (TestSuite *suite)
 {
-   dollar_query_test_t *p = tests;
-
-   while (p->name) {
-      TestSuite_AddFull (suite, p->name, _test_dollar_query, NULL, p, TestSuite_CheckMockServerAllowed);
-
-      p++;
-   }
-
    TestSuite_AddLive (suite, "/Cluster/test_get_max_bson_obj_size", test_get_max_bson_obj_size);
    TestSuite_AddLive (suite, "/Cluster/test_get_max_msg_size", test_get_max_msg_size);
    TestSuite_AddFull (suite,
@@ -1807,8 +1568,6 @@ test_cluster_install (TestSuite *suite)
                       test_framework_skip_if_slow);
    TestSuite_AddLive (suite, "/Cluster/cluster_time/command_simple/single", test_cluster_time_command_simple_single);
    TestSuite_AddLive (suite, "/Cluster/cluster_time/command_simple/pooled", test_cluster_time_command_simple_pooled);
-   TestSuite_AddLive (suite, "/Cluster/cluster_time/command/single", test_cluster_time_command_single);
-   TestSuite_AddLive (suite, "/Cluster/cluster_time/command/pooled", test_cluster_time_command_pooled);
    TestSuite_AddLive (
       suite, "/Cluster/cluster_time/command_with_opts/single", test_cluster_time_command_with_opts_single);
    TestSuite_AddLive (
