@@ -13,7 +13,7 @@
 
 #include "json-test.h"
 #include "test-libmongoc.h"
-#include <common-cmp-private.h>
+#include <mlib/cmp.h>
 
 static void
 _assert_options_match (const bson_t *test, mongoc_uri_t *uri)
@@ -203,7 +203,7 @@ _host_list_matches (const bson_t *test, context_t *ctx)
       ctx->hosts = NULL;
       bson_mutex_unlock (&ctx->mutex);
 
-      ret = mcommon_cmp_equal_su (expected, actual);
+      ret = mlib_cmp (expected, ==, actual);
    }
 
    return ret;
@@ -908,7 +908,7 @@ _mock_rr_resolver_prose_test_10 (const char *service,
 
    if (rr_type == MONGOC_RR_SRV) {
       const size_t count = _mongoc_host_list_length (rr_data->hosts);
-      BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, count));
+      BSON_ASSERT (mlib_in_range (uint32_t, count));
 
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27017",
                                    "localhost.test.build.10gen.cc:27019",
@@ -1010,7 +1010,7 @@ _mock_rr_resolver_prose_test_11 (const char *service,
 
    if (rr_type == MONGOC_RR_SRV) {
       const size_t count = _mongoc_host_list_length (rr_data->hosts);
-      BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, count));
+      BSON_ASSERT (mlib_in_range (uint32_t, count));
 
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27019", "localhost.test.build.10gen.cc:27020");
       rr_data->count = (uint32_t) count;
@@ -1109,7 +1109,7 @@ _mock_rr_resolver_prose_test_12 (const char *service,
 
    if (rr_type == MONGOC_RR_SRV) {
       const size_t count = _mongoc_host_list_length (rr_data->hosts);
-      BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, count));
+      BSON_ASSERT (mlib_in_range (uint32_t, count));
 
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27017",
                                    "localhost.test.build.10gen.cc:27019",
@@ -1260,7 +1260,7 @@ _mock_rr_resolver_with_override (const char *service,
    if (rr_type == MONGOC_RR_SRV) {
       bson_mutex_lock (&rr_override.lock);
       const size_t count = _mongoc_host_list_length (rr_override.hosts);
-      BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, count));
+      BSON_ASSERT (mlib_in_range (uint32_t, count));
       rr_data->hosts = _mongoc_host_list_copy_all (rr_override.hosts);
       rr_data->count = (uint32_t) count;
       rr_data->txt_record_opts = NULL;
