@@ -9,7 +9,7 @@
 #include "mock_server/future.h"
 #include "mock_server/future-functions.h"
 #include "json-test-monitoring.h"
-#include <common-cmp-private.h>
+#include <mlib/cmp.h>
 
 #ifdef BSON_HAVE_STRINGS_H
 #include <strings.h>
@@ -135,7 +135,7 @@ td_to_bson (const mongoc_topology_description_t *td, bson_t *bson)
    mongoc_set_t const *servers_set = mc_tpld_servers_const (td);
 
    for (size_t i = 0; i < servers_set->items_len; i++) {
-      BSON_ASSERT (mcommon_in_range_unsigned (uint32_t, i));
+      BSON_ASSERT (mlib_in_range (uint32_t, i));
       bson_uint32_to_string ((uint32_t) i, &key, str, sizeof str);
       sd_to_bson (mongoc_set_get_item_const (servers_set, i), &server);
       BSON_APPEND_DOCUMENT (&servers, key, &server);
