@@ -10,7 +10,7 @@
 #include "test-libmongoc.h"
 #include "test-conveniences.h"
 
-#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && !defined(MONGOC_ENABLE_SSL_LIBRESSL)
+#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL)
 
 static void
 test_mongoc_tls_no_certs (void)
@@ -387,7 +387,7 @@ test_mongoc_tls_trust_dir (void)
 }
 #endif
 
-#endif /* !MONGOC_ENABLE_SSL_SECURE_CHANNEL && !MONGOC_ENABLE_SSL_LIBRESSL */
+#endif /* !MONGOC_ENABLE_SSL_SECURE_CHANNEL */
 
 void
 test_mongoc_tls_insecure_nowarning (void)
@@ -413,7 +413,7 @@ test_mongoc_tls_insecure_nowarning (void)
 void
 test_stream_tls_install (TestSuite *suite)
 {
-#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && !defined(MONGOC_ENABLE_SSL_LIBRESSL)
+#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL)
 
    /* Disable /TLS/commonName on macOS due to CDRIVER-4256. */
 #if !defined(MONGOC_ENABLE_SSL_SECURE_TRANSPORT)
